defmodule WhiteBreadContext.Restaurant do
  use WhiteBread.Context
  use Hound.Helpers
  import Volt.AccountsFixtures
  alias Volt.{Repo, Sales.Order}
  import VoltWeb.RestaurantAuth

  feature_starting_state fn  ->
    Application.ensure_all_started(:hound)
    %{}
  end
  scenario_starting_state fn _state ->
    Hound.start_session
    Ecto.Adapters.SQL.Sandbox.checkout(Volt.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(Volt.Repo, {:shared, self()})

    %{}
  end
  scenario_finalize fn _status, _state ->
    Hound.end_session
    Ecto.Adapters.SQL.Sandbox.checkin(Volt.Repo)
  end

  and_ ~r/^I open restaurant registration page$/, fn state ->
    navigate_to "/restaurants/register"
    {:ok, state}
  end

  and_ ~r/^I enter the restaurant information$/, fn state ->
    fill_field({:id, "restaurant_email"}, "giorgio_cuisine@restaurant.gov")
    fill_field({:id, "restaurant_password"}, "qwerty123uyghyiu")
    fill_field({:id, "restaurant_first_name"}, "Giorgio")
    fill_field({:id, "restaurant_last_name"}, "Shumaylovi")
    fill_field({:id, "restaurant_phone_number"}, "+37253584669")
    fill_field({:id, "restaurant_name"}, "Shit's Bussin")
    fill_field({:id, "restaurant_address"}, "Joe Mama 69")
    fill_field({:id, "restaurant_city"}, "Tartu")
    fill_field({:id, "restaurant_zip_code"}, "51004")
    fill_field({:id, "restaurant_price_level"}, 1)
    {:ok, state}
  end

  when_ ~r/^I submit the form$/, fn state ->
    submit_element({:id, "submit"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to the restaurant dashboard$/, fn state ->
    {:ok, state}
  end

  given_ ~r/^that a restaurant worker is connected$/, fn state ->
    log_in_restaurant(state, restaurant_fixture())
    {:ok, state}
  end

  given_ ~r/^that I want to add this item: "(?<name>[^"]+)", "(?<description>[^"]+)", "(?<unit_price>[^"]+)"$/,
  fn state, %{name: name, description: description, unit_price: unit_price} ->
    {:ok, state |> Map.put(:name, name) |> Map.put(:description, description) |> Map.put(:unit_price, unit_price) |> Map.put(:discount, "0")}
  end

  given_ ~r/^that I want to add this item with discount: "(?<name>[^"]+)", "(?<description>[^"]+)", "(?<unit_price>[^"]+)", "(?<discount>[^"]+)"$/,
  fn state, %{name: name, description: description, unit_price: unit_price, discount: discount} ->
    {:ok, state |> Map.put(:name, name) |> Map.put(:description, description) |> Map.put(:unit_price, unit_price) |> Map.put(:discount, discount)}
  end

  then_ ~r/^I open the page to add an item$/, fn state ->
    navigate_to "/restaurants/additem"
    {:ok, state}
  end

  and_ ~r/^I fill the form$/, fn state ->
    fill_field({:id, "item_name"}, state[:name])
    fill_field({:id, "item_description"}, state[:description])
    fill_field({:id, "item_unit_price"}, state[:unit_price])
    fill_field({:id, "item_discount"}, state[:discount])
    {:ok, state}
  end

  and_ ~r/^I fill the form with error$/, fn state ->
    fill_field({:id, "item_name"}, "h")
    fill_field({:id, "item_description"}, "")
    fill_field({:id, "item_unit_price"}, "s")
    {:ok, state}
  end

  then_ ~r/^I open menu page$/, fn state ->
    navigate_to "/restaurants/menu"
    {:ok, state}
  end

  when_ ~r/^I click delete the item$/, fn state ->
    click({:id, "delete"})
    {:ok, state}
  end

  and_ ~r/^I click edit the item$/, fn state ->
    click({:id, "edit"})
    {:ok, state}
  end

  then_ ~r/^I should have a confirmation that item has been created$/, fn state ->
    assert visible_in_page? ~r/Item created successfully./
    {:ok, state}
  end

  then_ ~r/^I should have a confirmation for editing item$/, fn state ->
    assert visible_in_page? ~r/Item updated successfully./
    {:ok, state}
  end

  then_ ~r/^I should have a confirmation for deleting$/, fn state ->
    assert visible_in_page? ~r/Item deleted successfully./
    {:ok, state}
  end

  given_ ~r/^that customer, restaurant and order are initialised$/, fn state  ->
    customer = customer_fixture(%{
      address: "Raatuse 22",
      balance: 0.00,
      birth_date: ~D[2022-11-04],
      card_number: "123123123123123",
      email: "roro@gmail.com",
      password: "roromain12345*",
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: "+33641941064",
      city: "Tartu",
      zip_code: "51009"
    })
    restaurant = restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "kolmtilli@gmail.com",
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: "kolmtilli12345*",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 1
    })
    order = %Order{order_status: "PENDING", order_price: 10.00, total_price: 12.50}
    order = Ecto.build_assoc(restaurant, :orders, order)
    order = Ecto.build_assoc(customer, :orders, order)
    case Repo.insert(order) do
      {:ok, _order} ->
        {:ok, state}
      {:error, %Ecto.Changeset{} = _changeset} ->
        {:error, state}
    end
  end

  given_ ~r/^that customer, restaurant, order and its courier are initialised$/, fn state  ->
    customer = customer_fixture(%{
      address: "Raatuse 22",
      balance: 0.00,
      birth_date: ~D[2022-11-04],
      card_number: "123123123123123",
      email: "roro@gmail.com",
      password: "roromain12345*",
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: "+33641941064",
      city: "Tartu",
      zip_code: "51009"
    })
    restaurant = restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "kolmtilli@gmail.com",
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: "kolmtilli12345*",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 1
    })
    courier = courier_fixture()

    prep_duration = 10
    prep_time = NaiveDateTime.add(NaiveDateTime.local_now(), prep_duration*60, :second)
    del_duration = 20
    del_time = NaiveDateTime.add(prep_time, del_duration*60, :second)

    order = %Order{order_status: "IN_PROCESS", order_price: 10.00, total_price: 12.50, preparation_duration: prep_duration, preparation_time: prep_time, delivery_duration: del_duration, delivery_time: del_time}
    order = Ecto.build_assoc(restaurant, :orders, order)
    order = Ecto.build_assoc(customer, :orders, order)
    order = Ecto.build_assoc(courier, :orders, order)
    case Repo.insert(order) do
      {:ok, _order} ->
        {:ok, state}
      {:error, %Ecto.Changeset{} = _changeset} ->
        {:error, state}
    end
  end

  and_ ~r/^the restaurant logins$/, fn state ->
    navigate_to "/restaurants/log_in"
    fill_field({:id, "restaurant_email"}, "kolmtilli@gmail.com")
    fill_field({:id, "restaurant_password"}, "kolmtilli12345*")
    click({:id, "submit_button"})
    {:ok, state}
  end

  when_ ~r/^the restaurant access his dashboard and click the cancel button for this order$/, fn state ->
    click({:id, "cancel_pending"})
    {:ok, state}
  end

  then_ ~r/^the restaurant enter a message$/, fn state ->
    fill_field({:id, "restaurant_message"}, "My random message")
    {:ok, state}
  end

  when_ ~r/^he clicks the submit button$/, fn state ->
    click({:id, "submit"})
    {:ok, state}
  end

  then_ ~r/^he should be redirected to dashboard and have a confirmation for cancelling$/, fn state ->
    assert visible_in_page? ~r/Order canceled successfully./
    {:ok, state}
  end

  then_ ~r/^he should have a confirmation for changing preparation time$/, fn state ->
    assert visible_in_page? ~r/Preparation time edited successfully./
    {:ok, state}
  end

  then_ ~r/^he should have an error message because message is needed$/, fn state ->
    assert visible_in_page? ~r/You must provide a message/
    {:ok, state}
  end

  when_ ~r/^the restaurant access his dashboard and click the accept button for this order$/, fn state ->
    click({:id, "accept_pending"})
    {:ok, state}
  end

  then_ ~r/^the restaurant enter a preparation duration estimation in minute$/, fn state ->
    fill_field({:id, "preparation_duration"}, 10)
    {:ok, state}
  end

  then_ ~r/^he should be redirected to dashboard and have a confirmation for accepting$/, fn state ->
    assert visible_in_page? ~r/Order accepted successfully./
    {:ok, state}
  end

  and_ ~r/^the restaurant access his dashboard and click the manage button for this order$/, fn state ->
    click({:id, "manage"})
    {:ok, state}
  end

  when_ ~r/^he clicks the mark as prepared button$/, fn state ->
    click({:id, "prepare"})
    {:ok, state}
  end

  then_ ~r/^he should have a confirmation for preparing$/, fn state ->
    assert visible_in_page? ~r/Order marked as prepared successfully./
    {:ok, state}
  end

  then_ ~r/^he should have an error message because courier is missing$/, fn state ->
    assert visible_in_page? ~r/Order has to have a courier first!/
    {:ok, state}
  end

  given_ ~r/^that restaurant is initialised$/, fn state ->
    restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "kolmtilli@gmail.com",
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: "kolmtilli12345*",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 1
    })
    {:ok, state}
  end

  and_ ~r/^I enter the restaurant information with error$/, fn state ->
    fill_field({:id, "restaurant_email"}, "giorgio_cuisine@restaurant.gov")
    fill_field({:id, "restaurant_password"}, "qwerty123uyghyiu")
    fill_field({:id, "restaurant_first_name"}, "Giorgio")
    fill_field({:id, "restaurant_last_name"}, "Shumaylovi")
    fill_field({:id, "restaurant_phone_number"}, "+37253584669")
    fill_field({:id, "restaurant_name"}, "Shit's Bussin")
    fill_field({:id, "restaurant_address"}, "Joe Mama 69")
    fill_field({:id, "restaurant_city"}, "Tartu")
    fill_field({:id, "restaurant_zip_code"}, "51004TYUII")
    fill_field({:id, "restaurant_price_level"}, 1)
    {:ok, state}
  end

  then_ ~r/^he should be redirected to dashboard with confirmation$/, fn state ->
    assert visible_in_page? ~r/Restaurant Dashboard/
    {:ok, state}
  end

  and_ ~r/^the restaurant logins with error$/, fn state ->
    navigate_to "/restaurants/log_in"
    fill_field({:id, "restaurant_email"}, "kolmtilli@gmail.com")
    fill_field({:id, "restaurant_password"}, "wrongpassword2345ghj")
    click({:id, "submit_button"})
    {:ok, state}
  end

  when_ ~r/^he clicks the logout button$/, fn state ->
    click({:id, "logout"})
    {:ok, state}
  end

  then_ ~r/^he should see an error for login$/, fn state ->
    assert visible_in_page? ~r/Invalid email or password/
    {:ok, state}
  end

  then_ ~r/^he should see a logout confirmation$/, fn state ->
    assert visible_in_page? ~r/Logged out successfully./
    {:ok, state}
  end

  then_ ~r/^I should have a error message while completing the form$/, fn state ->
    assert visible_in_page? ~r/Oops, something went wrong! Please check the errors below./
    {:ok, state}
  end
end
