defmodule Volt.Accounts do
  @moduledoc """
  The Accounts context.
  """

  import Ecto.Query, warn: false
  alias Volt.Repo

  alias Volt.Accounts.Restaurant

  @doc """
  Returns the list of restaurants.

  ## Examples

      iex> list_restaurants()
      [%Restaurant{}, ...]

  """
  def list_restaurants do
    Repo.all(Restaurant)
  end

  alias Volt.Accounts.Courier

  @doc """
  Returns the list of couriers.

  ## Examples

      iex> list_couriers()
      [%Courier{}, ...]

  """
  def list_couriers do
    Repo.all(Courier)
  end

  @doc """
  Gets a single courier.

  Raises `Ecto.NoResultsError` if the Courier does not exist.

  ## Examples

      iex> get_courier!(123)
      %Courier{}

      iex> get_courier!(456)
      ** (Ecto.NoResultsError)

  """
  def get_courier!(id), do: Repo.get!(Courier, id)

  alias Volt.Accounts.Customer

  @doc """
  Returns the list of customers.

  ## Examples

      iex> list_customers()
      [%Customer{}, ...]

  """
  def list_customers do
    Repo.all(Customer)
  end

  alias Volt.Accounts.{Courier, CourierToken, CourierNotifier}

  ## Database getters

  @doc """
  Gets a courier by email.

  ## Examples

      iex> get_courier_by_email("foo@example.com")
      %Courier{}

      iex> get_courier_by_email("unknown@example.com")
      nil

  """
  def get_courier_by_email(email) when is_binary(email) do
    Repo.get_by(Courier, email: email)
  end

  @doc """
  Gets a courier by email and password.

  ## Examples

      iex> get_courier_by_email_and_password("foo@example.com", "correct_password")
      %Courier{}

      iex> get_courier_by_email_and_password("foo@example.com", "invalid_password")
      nil

  """
  def get_courier_by_email_and_password(email, password)
      when is_binary(email) and is_binary(password) do
    courier = Repo.get_by(Courier, email: email)
    if Courier.valid_password?(courier, password), do: courier
  end

  ## Courier registration

  @doc """
  Registers a courier.

  ## Examples

      iex> register_courier(%{field: value})
      {:ok, %Courier{}}

      iex> register_courier(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def register_courier(attrs) do
    %Courier{}
    |> Courier.registration_changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking courier changes.

  ## Examples

      iex> change_courier_registration(courier)
      %Ecto.Changeset{data: %Courier{}}

  """
  def change_courier_registration(%Courier{} = courier, attrs \\ %{}) do
    Courier.registration_changeset(courier, attrs, hash_password: false)
  end

  ## Settings

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the courier email.

  ## Examples

      iex> change_courier_email(courier)
      %Ecto.Changeset{data: %Courier{}}

  """
  def change_courier_email(courier, attrs \\ %{}) do
    Courier.email_changeset(courier, attrs)
  end

  @doc """
  Emulates that the email will change without actually changing
  it in the database.

  ## Examples

      iex> apply_courier_email(courier, "valid password", %{email: ...})
      {:ok, %Courier{}}

      iex> apply_courier_email(courier, "invalid password", %{email: ...})
      {:error, %Ecto.Changeset{}}

  """
  def apply_courier_email(courier, password, attrs) do
    courier
    |> Courier.email_changeset(attrs)
    |> Courier.validate_current_password(password)
    |> Ecto.Changeset.apply_action(:update)
  end

  @doc """
  Updates the courier email using the given token.

  If the token matches, the courier email is updated and the token is deleted.
  The confirmed_at date is also updated to the current time.
  """
  def update_courier_email(courier, token) do
    context = "change:#{courier.email}"

    with {:ok, query} <- CourierToken.verify_change_email_token_query(token, context),
         %CourierToken{sent_to: email} <- Repo.one(query),
         {:ok, _} <- Repo.transaction(courier_email_multi(courier, email, context)) do
      :ok
    else
      _ -> :error
    end
  end

  defp courier_email_multi(courier, email, context) do
    changeset =
      courier
      |> Courier.email_changeset(%{email: email})
      |> Courier.confirm_changeset()

    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, changeset)
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, [context]))
  end

  @doc """
  Delivers the update email instructions to the given courier.

  ## Examples

      iex> deliver_courier_update_email_instructions(courier, current_email, &Routes.courier_update_email_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_courier_update_email_instructions(%Courier{} = courier, current_email, update_email_url_fun)
      when is_function(update_email_url_fun, 1) do
    {encoded_token, courier_token} = CourierToken.build_email_token(courier, "change:#{current_email}")

    Repo.insert!(courier_token)
    CourierNotifier.deliver_courier_update_email_instructions(courier, update_email_url_fun.(encoded_token))
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the courier password.

  ## Examples

      iex> change_courier_password(courier)
      %Ecto.Changeset{data: %Courier{}}

  """
  def change_courier_password(courier, attrs \\ %{}) do
    Courier.password_changeset(courier, attrs, hash_password: false)
  end

  @doc """
  Updates the courier password.

  ## Examples

      iex> update_courier_password(courier, "valid password", %{password: ...})
      {:ok, %Courier{}}

      iex> update_courier_password(courier, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_courier_password(courier, password, attrs) do
    changeset =
      courier
      |> Courier.password_changeset(attrs)
      |> Courier.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, changeset)
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{courier: courier}} -> {:ok, courier}
      {:error, :courier, changeset, _} -> {:error, changeset}
    end
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the courier profile.

  ## Examples

      iex> change_courier_profile(courier)
      %Ecto.Changeset{data: %Courier{}}

  """
  def change_courier_profile(courier, attrs \\ %{}) do
    Courier.profile_changeset(courier, attrs)
  end

  @doc """
  Updates the courier profile.

  ## Examples

      iex> update_courier_profile(courier, "valid password", %{password: ...})
      {:ok, %Courier{}}

      iex> update_courier_profile(courier, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_courier_profile(courier, password, attrs) do
    changeset =
      courier
      |> Courier.profile_changeset(attrs)
      |> Courier.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, changeset)
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, :all))
    |> Repo.transaction()
    |> case do
         {:ok, %{courier: courier}} -> {:ok, courier}
         {:error, :courier, changeset, _} -> {:error, changeset}
       end
  end

 @doc """
  Returns an `%Ecto.Changeset{}` for changing the courier status.

  ## Examples

      iex> change_courier_status(courier)
      %Ecto.Changeset{data: %Courier{}}

  """
  def change_courier_status(courier, attrs \\ %{}) do
    Courier.status_changeset(courier, attrs)
  end

  @doc """
  Updates the courier status.

  ## Examples

      iex> update_courier_status(courier, "valid password", %{password: ...})
      {:ok, %Courier{}}

      iex> update_courier_status(courier, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_courier_status(courier, password, attrs) do
    changeset =
    courier
    |> Courier.status_changeset(attrs)
    |> Courier.validate_current_password(password)


    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, changeset)
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{courier: courier}} -> {:ok, courier}
      {:error, :courier, changeset, _} -> {:error, changeset}
    end
  end

  ## Session

  @doc """
  Generates a session token.
  """
  def generate_courier_session_token(courier) do
    {token, courier_token} = CourierToken.build_session_token(courier)
    Repo.insert!(courier_token)
    token
  end

  @doc """
  Gets the courier with the given signed token.
  """
  def get_courier_by_session_token(token) do
    {:ok, query} = CourierToken.verify_session_token_query(token)
    Repo.one(query)
  end

  @doc """
  Deletes the signed token with the given context.
  """
  def delete_courier_session_token(token) do
    Repo.delete_all(CourierToken.token_and_context_query(token, "session"))
    :ok
  end

  ## Confirmation

  @doc """
  Delivers the confirmation email instructions to the given courier.

  ## Examples

      iex> deliver_courier_confirmation_instructions(courier, &Routes.courier_confirmation_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

      iex> deliver_courier_confirmation_instructions(confirmed_courier, &Routes.courier_confirmation_url(conn, :edit, &1))
      {:error, :already_confirmed}

  """
  def deliver_courier_confirmation_instructions(%Courier{} = courier, confirmation_url_fun)
      when is_function(confirmation_url_fun, 1) do
    if courier.confirmed_at do
      {:error, :already_confirmed}
    else
      {encoded_token, courier_token} = CourierToken.build_email_token(courier, "confirm")
      Repo.insert!(courier_token)
      CourierNotifier.deliver_confirmation_instructions(courier, confirmation_url_fun.(encoded_token))
    end
  end

  @doc """
  Confirms a courier by the given token.

  If the token matches, the courier account is marked as confirmed
  and the token is deleted.
  """
  def confirm_courier(token) do
    with {:ok, query} <- CourierToken.verify_email_token_query(token, "confirm"),
         %Courier{} = courier <- Repo.one(query),
         {:ok, %{courier: courier}} <- Repo.transaction(confirm_courier_multi(courier)) do
      {:ok, courier}
    else
      _ -> :error
    end
  end

  defp confirm_courier_multi(courier) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, Courier.confirm_changeset(courier))
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, ["confirm"]))
  end

  ## Reset password

  @doc """
  Delivers the reset password email to the given courier.

  ## Examples

      iex> deliver_courier_reset_password_instructions(courier, &Routes.courier_reset_password_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_courier_reset_password_instructions(%Courier{} = courier, reset_password_url_fun)
      when is_function(reset_password_url_fun, 1) do
    {encoded_token, courier_token} = CourierToken.build_email_token(courier, "reset_password")
    Repo.insert!(courier_token)
    CourierNotifier.deliver_reset_password_instructions(courier, reset_password_url_fun.(encoded_token))
  end

  @doc """
  Gets the courier by reset password token.

  ## Examples

      iex> get_courier_by_reset_password_token("validtoken")
      %Courier{}

      iex> get_courier_by_reset_password_token("invalidtoken")
      nil

  """
  def get_courier_by_reset_password_token(token) do
    with {:ok, query} <- CourierToken.verify_email_token_query(token, "reset_password"),
         %Courier{} = courier <- Repo.one(query) do
      courier
    else
      _ -> nil
    end
  end

  @doc """
  Resets the courier password.

  ## Examples

      iex> reset_courier_password(courier, %{password: "new long password", password_confirmation: "new long password"})
      {:ok, %Courier{}}

      iex> reset_courier_password(courier, %{password: "valid", password_confirmation: "not the same"})
      {:error, %Ecto.Changeset{}}

  """
  def reset_courier_password(courier, attrs) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:courier, Courier.password_changeset(courier, attrs))
    |> Ecto.Multi.delete_all(:tokens, CourierToken.courier_and_contexts_query(courier, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{courier: courier}} -> {:ok, courier}
      {:error, :courier, changeset, _} -> {:error, changeset}
    end
  end

  alias Volt.Accounts.{Restaurant, RestaurantToken, RestaurantNotifier}

  ## Database getters

  @doc """
  Gets a restaurant by email.

  ## Examples

      iex> get_restaurant_by_email("foo@example.com")
      %Restaurant{}

      iex> get_restaurant_by_email("unknown@example.com")
      nil

  """
  def get_restaurant_by_email(email) when is_binary(email) do
    Repo.get_by(Restaurant, email: email)
  end

  @doc """
  Gets a restaurant by email and password.

  ## Examples

      iex> get_restaurant_by_email_and_password("foo@example.com", "correct_password")
      %Restaurant{}

      iex> get_restaurant_by_email_and_password("foo@example.com", "invalid_password")
      nil

  """
  def get_restaurant_by_email_and_password(email, password)
      when is_binary(email) and is_binary(password) do
    restaurant = Repo.get_by(Restaurant, email: email)
    if Restaurant.valid_password?(restaurant, password), do: restaurant
  end

  @doc """
  Gets a single restaurant.

  Raises `Ecto.NoResultsError` if the Restaurant does not exist.

  ## Examples

      iex> get_restaurant!(123)
      %Restaurant{}

      iex> get_restaurant!(456)
      ** (Ecto.NoResultsError)

  """
  def get_restaurant!(id), do: Repo.get!(Restaurant, id)

  ## Restaurant registration

  @doc """
  Registers a restaurant.

  ## Examples

      iex> register_restaurant(%{field: value})
      {:ok, %Restaurant{}}

      iex> register_restaurant(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def register_restaurant(attrs) do
    %Restaurant{}
    |> Restaurant.registration_changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking restaurant changes.

  ## Examples

      iex> change_restaurant_registration(restaurant)
      %Ecto.Changeset{data: %Restaurant{}}

  """
  def change_restaurant_registration(%Restaurant{} = restaurant, attrs \\ %{}) do
    Restaurant.registration_changeset(restaurant, attrs, hash_password: false)
  end

  ## Settings

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the restaurant email.

  ## Examples

      iex> change_restaurant_email(restaurant)
      %Ecto.Changeset{data: %Restaurant{}}

  """
  def change_restaurant_email(restaurant, attrs \\ %{}) do
    Restaurant.email_changeset(restaurant, attrs)
  end

  @doc """
  # TODO: email should be changed for real
  Emulates that the email will change without actually changing
  it in the database.

  ## Examples

      iex> apply_restaurant_email(restaurant, "valid password", %{email: ...})
      {:ok, %Restaurant{}}

      iex> apply_restaurant_email(restaurant, "invalid password", %{email: ...})
      {:error, %Ecto.Changeset{}}

  """
  def apply_restaurant_email(restaurant, password, attrs) do
    restaurant
    |> Restaurant.email_changeset(attrs)
    |> Restaurant.validate_current_password(password)
    |> Ecto.Changeset.apply_action(:update)
  end

  @doc """
  Updates the restaurant email using the given token.

  If the token matches, the restaurant email is updated and the token is deleted.
  The confirmed_at date is also updated to the current time.
  """
  def update_restaurant_email(restaurant, token) do
    context = "change:#{restaurant.email}"

    with {:ok, query} <- RestaurantToken.verify_change_email_token_query(token, context),
         %RestaurantToken{sent_to: email} <- Repo.one(query),
         {:ok, _} <- Repo.transaction(restaurant_email_multi(restaurant, email, context)) do
      :ok
    else
      _ -> :error
    end
  end

  defp restaurant_email_multi(restaurant, email, context) do
    changeset =
      restaurant
      |> Restaurant.email_changeset(%{email: email})
      |> Restaurant.confirm_changeset()

    Ecto.Multi.new()
    |> Ecto.Multi.update(:restaurant, changeset)
    |> Ecto.Multi.delete_all(:tokens, RestaurantToken.restaurant_and_contexts_query(restaurant, [context]))
  end

  @doc """
  Delivers the update email instructions to the given restaurant.

  ## Examples

      iex> deliver_restaurant_update_email_instructions(restaurant, current_email, &Routes.restaurant_update_email_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_restaurant_update_email_instructions(%Restaurant{} = restaurant, current_email, update_email_url_fun)
      when is_function(update_email_url_fun, 1) do
    {encoded_token, restaurant_token} = RestaurantToken.build_email_token(restaurant, "change:#{current_email}")

    Repo.insert!(restaurant_token)
    RestaurantNotifier.deliver_restaurant_update_email_instructions(restaurant, update_email_url_fun.(encoded_token))
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the restaurant profile.

  ## Examples

      iex> change_restaurant_profile(restaurant)
      %Ecto.Changeset{data: %Restaurant{}}

  """
  def change_restaurant_profile(restaurant, attrs \\ %{}) do
    Restaurant.profile_changeset(restaurant, attrs)
  end

  @doc """
  Updates the restaurant profile.

  ## Examples

      iex> update_restaurant_profile(restaurant, "valid password", %{password: ...})
      {:ok, %Restaurant{}}

      iex> update_restaurant_profile(restaurant, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_restaurant_profile(restaurant, password, attrs) do
    changeset =
      restaurant
      |> Restaurant.profile_changeset(attrs)
      |> Restaurant.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:restaurant, changeset)
    |> Ecto.Multi.delete_all(:tokens, RestaurantToken.restaurant_and_contexts_query(restaurant, :all))
    |> Repo.transaction()
    |> case do
         {:ok, %{restaurant: restaurant}} -> {:ok, restaurant}
         {:error, :restaurant, changeset, _} -> {:error, changeset}
       end
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the restaurant password.

  ## Examples

      iex> change_restaurant_password(restaurant)
      %Ecto.Changeset{data: %Restaurant{}}

  """
  def change_restaurant_password(restaurant, attrs \\ %{}) do
    Restaurant.password_changeset(restaurant, attrs, hash_password: false)
  end

  @doc """
  Updates the restaurant password.

  ## Examples

      iex> update_restaurant_password(restaurant, "valid password", %{password: ...})
      {:ok, %Restaurant{}}

      iex> update_restaurant_password(restaurant, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_restaurant_password(restaurant, password, attrs) do
    changeset =
      restaurant
      |> Restaurant.password_changeset(attrs)
      |> Restaurant.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:restaurant, changeset)
    |> Ecto.Multi.delete_all(:tokens, RestaurantToken.restaurant_and_contexts_query(restaurant, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{restaurant: restaurant}} -> {:ok, restaurant}
      {:error, :restaurant, changeset, _} -> {:error, changeset}
    end
  end

  ## Session

  @doc """
  Generates a session token.
  """
  def generate_restaurant_session_token(restaurant) do
    {token, restaurant_token} = RestaurantToken.build_session_token(restaurant)
    Repo.insert!(restaurant_token)
    token
  end

  @doc """
  Gets the restaurant with the given signed token.
  """
  def get_restaurant_by_session_token(token) do
    {:ok, query} = RestaurantToken.verify_session_token_query(token)
    Repo.one(query)
  end

  @doc """
  Deletes the signed token with the given context.
  """
  def delete_restaurant_session_token(token) do
    Repo.delete_all(RestaurantToken.token_and_context_query(token, "session"))
    :ok
  end

  ## Confirmation

  @doc """
  Delivers the confirmation email instructions to the given restaurant.

  ## Examples

      iex> deliver_restaurant_confirmation_instructions(restaurant, &Routes.restaurant_confirmation_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

      iex> deliver_restaurant_confirmation_instructions(confirmed_restaurant, &Routes.restaurant_confirmation_url(conn, :edit, &1))
      {:error, :already_confirmed}

  """
  def deliver_restaurant_confirmation_instructions(%Restaurant{} = restaurant, confirmation_url_fun)
      when is_function(confirmation_url_fun, 1) do
    if restaurant.confirmed_at do
      {:error, :already_confirmed}
    else
      {encoded_token, restaurant_token} = RestaurantToken.build_email_token(restaurant, "confirm")
      Repo.insert!(restaurant_token)
      RestaurantNotifier.deliver_confirmation_instructions(restaurant, confirmation_url_fun.(encoded_token))
    end
  end

  @doc """
  Confirms a restaurant by the given token.

  If the token matches, the restaurant account is marked as confirmed
  and the token is deleted.
  """
  def confirm_restaurant(token) do
    with {:ok, query} <- RestaurantToken.verify_email_token_query(token, "confirm"),
         %Restaurant{} = restaurant <- Repo.one(query),
         {:ok, %{restaurant: restaurant}} <- Repo.transaction(confirm_restaurant_multi(restaurant)) do
      {:ok, restaurant}
    else
      _ -> :error
    end
  end

  defp confirm_restaurant_multi(restaurant) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:restaurant, Restaurant.confirm_changeset(restaurant))
    |> Ecto.Multi.delete_all(:tokens, RestaurantToken.restaurant_and_contexts_query(restaurant, ["confirm"]))
  end

  ## Reset password

  @doc """
  Delivers the reset password email to the given restaurant.

  ## Examples

      iex> deliver_restaurant_reset_password_instructions(restaurant, &Routes.restaurant_reset_password_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_restaurant_reset_password_instructions(%Restaurant{} = restaurant, reset_password_url_fun)
      when is_function(reset_password_url_fun, 1) do
    {encoded_token, restaurant_token} = RestaurantToken.build_email_token(restaurant, "reset_password")
    Repo.insert!(restaurant_token)
    RestaurantNotifier.deliver_reset_password_instructions(restaurant, reset_password_url_fun.(encoded_token))
  end

  @doc """
  Gets the restaurant by reset password token.

  ## Examples

      iex> get_restaurant_by_reset_password_token("validtoken")
      %Restaurant{}

      iex> get_restaurant_by_reset_password_token("invalidtoken")
      nil

  """
  def get_restaurant_by_reset_password_token(token) do
    with {:ok, query} <- RestaurantToken.verify_email_token_query(token, "reset_password"),
         %Restaurant{} = restaurant <- Repo.one(query) do
      restaurant
    else
      _ -> nil
    end
  end

  @doc """
  Resets the restaurant password.

  ## Examples

      iex> reset_restaurant_password(restaurant, %{password: "new long password", password_confirmation: "new long password"})
      {:ok, %Restaurant{}}

      iex> reset_restaurant_password(restaurant, %{password: "valid", password_confirmation: "not the same"})
      {:error, %Ecto.Changeset{}}

  """
  def reset_restaurant_password(restaurant, attrs) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:restaurant, Restaurant.password_changeset(restaurant, attrs))
    |> Ecto.Multi.delete_all(:tokens, RestaurantToken.restaurant_and_contexts_query(restaurant, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{restaurant: restaurant}} -> {:ok, restaurant}
      {:error, :restaurant, changeset, _} -> {:error, changeset}
    end
  end

  alias Volt.Accounts.{Customer, CustomerToken, CustomerNotifier}

  ## Database getters

  @doc """
  Gets a customer by email.

  ## Examples

      iex> get_customer_by_email("foo@example.com")
      %Customer{}

      iex> get_customer_by_email("unknown@example.com")
      nil

  """
  def get_customer_by_email(email) when is_binary(email) do
    Repo.get_by(Customer, email: email)
  end

  @doc """
  Gets a customer by email and password.

  ## Examples

      iex> get_customer_by_email_and_password("foo@example.com", "correct_password")
      %Customer{}

      iex> get_customer_by_email_and_password("foo@example.com", "invalid_password")
      nil

  """
  def get_customer_by_email_and_password(email, password)
      when is_binary(email) and is_binary(password) do
    customer = Repo.get_by(Customer, email: email)
    if Customer.valid_password?(customer, password), do: customer
  end

  @doc """
  Gets a single customer.

  Raises `Ecto.NoResultsError` if the Customer does not exist.

  ## Examples

      iex> get_customer!(123)
      %Customer{}

      iex> get_customer!(456)
      ** (Ecto.NoResultsError)

  """
  def get_customer!(id), do: Repo.get!(Customer, id)

  ## Customer registration

  @doc """
  Registers a customer.

  ## Examples

      iex> register_customer(%{field: value})
      {:ok, %Customer{}}

      iex> register_customer(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def register_customer(attrs) do
    %Customer{}
    |> Customer.registration_changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking customer changes.

  ## Examples

      iex> change_customer_registration(customer)
      %Ecto.Changeset{data: %Customer{}}

  """
  def change_customer_registration(%Customer{} = customer, attrs \\ %{}) do
    Customer.registration_changeset(customer, attrs, hash_password: false)
  end

  ## Settings

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the customer profile.

  ## Examples

      iex> change_customer_profile(customer)
      %Ecto.Changeset{data: %Customer{}}

  """
  def change_customer_profile(customer, attrs \\ %{}) do
    Customer.profile_changeset(customer, attrs)
  end

  @doc """
  Updates the customer profile.

  ## Examples

      iex> update_customer_profile(customer, "valid password", %{password: ...})
      {:ok, %Customer{}}

      iex> update_customer_profile(customer, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_customer_profile(customer, password, attrs) do
    changeset =
      customer
      |> Customer.profile_changeset(attrs)
      |> Customer.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, changeset)
    |> Ecto.Multi.delete_all(:tokens, CustomerToken.customer_and_contexts_query(customer, :all))
    |> Repo.transaction()
    |> case do
         {:ok, %{customer: customer}} -> {:ok, customer}
         {:error, :customer, changeset, _} -> {:error, changeset}
       end
  end

  def update_customer_balance(customer, attrs) do
    changeset =
      customer
      |> Customer.profile_changeset(attrs)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, changeset)
    |> Repo.transaction()
    |> case do
         {:ok, %{customer: customer}} -> {:ok, customer}
         {:error, :customer, changeset, _} -> {:error, changeset}
       end
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the customer email.

  ## Examples

      iex> change_customer_email(customer)
      %Ecto.Changeset{data: %Customer{}}

  """
  def change_customer_email(customer, attrs \\ %{}) do
    Customer.email_changeset(customer, attrs)
  end

  @doc """
  Emulates that the email will change without actually changing
  it in the database.

  ## Examples

      iex> apply_customer_email(customer, "valid password", %{email: ...})
      {:ok, %Customer{}}

      iex> apply_customer_email(customer, "invalid password", %{email: ...})
      {:error, %Ecto.Changeset{}}

  """
  def apply_customer_email(customer, password, attrs) do
    customer
    |> Customer.email_changeset(attrs)
    |> Customer.validate_current_password(password)
    |> Ecto.Changeset.apply_action(:update)
  end

  @doc """
  Updates the customer email using the given token.

  If the token matches, the customer email is updated and the token is deleted.
  The confirmed_at date is also updated to the current time.
  """
  def update_customer_email(customer, token) do
    context = "change:#{customer.email}"

    with {:ok, query} <- CustomerToken.verify_change_email_token_query(token, context),
         %CustomerToken{sent_to: email} <- Repo.one(query),
         {:ok, _} <- Repo.transaction(customer_email_multi(customer, email, context)) do
      :ok
    else
      _ -> :error
    end
  end

  defp customer_email_multi(customer, email, context) do
    changeset =
      customer
      |> Customer.email_changeset(%{email: email})
      |> Customer.confirm_changeset()

    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, changeset)
    |> Ecto.Multi.delete_all(:tokens, CustomerToken.customer_and_contexts_query(customer, [context]))
  end

  @doc """
  Delivers the update email instructions to the given customer.

  ## Examples

      iex> deliver_customer_update_email_instructions(customer, current_email, &Routes.customer_update_email_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_customer_update_email_instructions(%Customer{} = customer, current_email, update_email_url_fun)
      when is_function(update_email_url_fun, 1) do
    {encoded_token, customer_token} = CustomerToken.build_email_token(customer, "change:#{current_email}")

    Repo.insert!(customer_token)
    CustomerNotifier.deliver_customer_update_email_instructions(customer, update_email_url_fun.(encoded_token))
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for changing the customer password.

  ## Examples

      iex> change_customer_password(customer)
      %Ecto.Changeset{data: %Customer{}}

  """
  def change_customer_password(customer, attrs \\ %{}) do
    Customer.password_changeset(customer, attrs, hash_password: false)
  end

  @doc """
  Updates the customer password.

  ## Examples

      iex> update_customer_password(customer, "valid password", %{password: ...})
      {:ok, %Customer{}}

      iex> update_customer_password(customer, "invalid password", %{password: ...})
      {:error, %Ecto.Changeset{}}

  """
  def update_customer_password(customer, password, attrs) do
    changeset =
      customer
      |> Customer.password_changeset(attrs)
      |> Customer.validate_current_password(password)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, changeset)
    |> Ecto.Multi.delete_all(:tokens, CustomerToken.customer_and_contexts_query(customer, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{customer: customer}} -> {:ok, customer}
      {:error, :customer, changeset, _} -> {:error, changeset}
    end
  end

  @doc """
  Updates the customer location.

  """
  def update_customer_location(customer, attrs \\ %{}) do
    changeset =
      customer
      |> Customer.location_changeset(attrs)

    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, changeset)
    |> Repo.transaction()
    customer
  end



  ## Session

  @doc """
  Generates a session token.
  """
  def generate_customer_session_token(customer) do
    {token, customer_token} = CustomerToken.build_session_token(customer)
    Repo.insert!(customer_token)
    token
  end

  @doc """
  Gets the customer with the given signed token.
  """
  def get_customer_by_session_token(token) do
    {:ok, query} = CustomerToken.verify_session_token_query(token)
    Repo.one(query)
  end

  @doc """
  Deletes the signed token with the given context.
  """
  def delete_customer_session_token(token) do
    Repo.delete_all(CustomerToken.token_and_context_query(token, "session"))
    :ok
  end

  ## Confirmation

  @doc """
  Delivers the confirmation email instructions to the given customer.

  ## Examples

      iex> deliver_customer_confirmation_instructions(customer, &Routes.customer_confirmation_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

      iex> deliver_customer_confirmation_instructions(confirmed_customer, &Routes.customer_confirmation_url(conn, :edit, &1))
      {:error, :already_confirmed}

  """
  def deliver_customer_confirmation_instructions(%Customer{} = customer, confirmation_url_fun)
      when is_function(confirmation_url_fun, 1) do
    if customer.confirmed_at do
      {:error, :already_confirmed}
    else
      {encoded_token, customer_token} = CustomerToken.build_email_token(customer, "confirm")
      Repo.insert!(customer_token)
      CustomerNotifier.deliver_confirmation_instructions(customer, confirmation_url_fun.(encoded_token))
    end
  end

  @doc """
  Confirms a customer by the given token.

  If the token matches, the customer account is marked as confirmed
  and the token is deleted.
  """
  def confirm_customer(token) do
    with {:ok, query} <- CustomerToken.verify_email_token_query(token, "confirm"),
         %Customer{} = customer <- Repo.one(query),
         {:ok, %{customer: customer}} <- Repo.transaction(confirm_customer_multi(customer)) do
      {:ok, customer}
    else
      _ -> :error
    end
  end

  defp confirm_customer_multi(customer) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, Customer.confirm_changeset(customer))
    |> Ecto.Multi.delete_all(:tokens, CustomerToken.customer_and_contexts_query(customer, ["confirm"]))
  end

  ## Reset password

  @doc """
  Delivers the reset password email to the given customer.

  ## Examples

      iex> deliver_customer_reset_password_instructions(customer, &Routes.customer_reset_password_url(conn, :edit, &1))
      {:ok, %{to: ..., body: ...}}

  """
  def deliver_customer_reset_password_instructions(%Customer{} = customer, reset_password_url_fun)
      when is_function(reset_password_url_fun, 1) do
    {encoded_token, customer_token} = CustomerToken.build_email_token(customer, "reset_password")
    Repo.insert!(customer_token)
    CustomerNotifier.deliver_reset_password_instructions(customer, reset_password_url_fun.(encoded_token))
  end

  @doc """
  Gets the customer by reset password token.

  ## Examples

      iex> get_customer_by_reset_password_token("validtoken")
      %Customer{}

      iex> get_customer_by_reset_password_token("invalidtoken")
      nil

  """
  def get_customer_by_reset_password_token(token) do
    with {:ok, query} <- CustomerToken.verify_email_token_query(token, "reset_password"),
         %Customer{} = customer <- Repo.one(query) do
      customer
    else
      _ -> nil
    end
  end

  @doc """
  Resets the customer password.

  ## Examples

      iex> reset_customer_password(customer, %{password: "new long password", password_confirmation: "new long password"})
      {:ok, %Customer{}}

      iex> reset_customer_password(customer, %{password: "valid", password_confirmation: "not the same"})
      {:error, %Ecto.Changeset{}}

  """
  def reset_customer_password(customer, attrs) do
    Ecto.Multi.new()
    |> Ecto.Multi.update(:customer, Customer.password_changeset(customer, attrs))
    |> Ecto.Multi.delete_all(:tokens, CustomerToken.customer_and_contexts_query(customer, :all))
    |> Repo.transaction()
    |> case do
      {:ok, %{customer: customer}} -> {:ok, customer}
      {:error, :customer, changeset, _} -> {:error, changeset}
    end
  end

  """
  TODO: Revise old tests

  def create_restaurant(attrs \\ %{}) do
    %Restaurant{}
    |> Restaurant.changeset(attrs)
    |> Repo.insert()
  end


  def update_restaurant(%Restaurant{} = restaurant, attrs) do
    restaurant
    |> Restaurant.changeset(attrs)
    |> Repo.update()
  end

  def delete_restaurant(%Restaurant{} = restaurant) do
    Repo.delete(restaurant)
  end


  def change_restaurant(%Restaurant{} = restaurant, attrs \\ %{}) do
    Restaurant.changeset(restaurant, attrs)
  end

  alias Volt.Accounts.Courier


  def list_couriers do
    Repo.all(Courier)
  end


  def get_courier!(id), do: Repo.get!(Courier, id)


  def create_courier(attrs \\ %{}) do
    %Courier{}
    |> Courier.changeset(attrs)
    |> Repo.insert()
  end


  def update_courier(%Courier{} = courier, attrs) do
    courier
    |> Courier.changeset(attrs)
    |> Repo.update()
  end


  def delete_courier(%Courier{} = courier) do
    Repo.delete(courier)
  end


  def change_courier(%Courier{} = courier, attrs \\ %{}) do
    Courier.changeset(courier, attrs)
  end

  alias Volt.Accounts.Customer


  def list_customers do
    Repo.all(Customer)
  end


  def create_customer(attrs \\ %{}) do
    %Customer{}
    |> Customer.changeset(attrs)
    |> Repo.insert()
  end


  def update_customer(%Customer{} = customer, attrs) do
    customer
    |> Customer.changeset(attrs)
    |> Repo.update()
  end


  def delete_customer(%Customer{} = customer) do
    Repo.delete(customer)
  end


  def change_customer(%Customer{} = customer, attrs \\ %{}) do
    Customer.changeset(customer, attrs)
  end
  """

end
