defmodule Volt.Accounts.Customer do
  use Ecto.Schema
  import Ecto.Changeset
  alias Volt.Sales
  schema "customers" do
    field :address, :string
    field :zip_code, :string
    field :city, :string
    field :balance, :float, default: 0.00
    field :birth_date, :date
    field :card_number, :string
    field :email, :string
    field :first_name, :string
    field :last_name, :string
    field :phone_number, :string
    field :password, :string, virtual: true, redact: true
    field :hashed_password, :string, redact: true
    field :confirmed_at, :naive_datetime
    has_many :orders, Sales.Order

    timestamps()
  end

  @doc """
  A customer changeset for registration.

  It is important to validate the length of both email and password.
  Otherwise databases may truncate the email without warnings, which
  could lead to unpredictable or insecure behaviour. Long passwords may
  also be very expensive to hash for certain algorithms.

  ## Options

    * `:hash_password` - Hashes the password so it can be stored securely
      in the database and ensures the password field is cleared to prevent
      leaks in the logs. If password hashing is not needed and clearing the
      password field is not desired (like when using this changeset for
      validations on a LiveView form), this option can be set to `false`.
      Defaults to `true`.
  """
  def registration_changeset(customer, attrs, opts \\ []) do
    customer
    |> cast(attrs, [:email, :password, :first_name, :last_name, :phone_number, :birth_date, :address, :zip_code, :city, :card_number, :balance])
    |> validate_profile()
    |> validate_email()
    |> validate_password(opts)
  end

  defp validate_profile(changeset) do
    changeset
    |> validate_required([:first_name, :last_name, :phone_number, :birth_date, :address, :zip_code, :city, :card_number, :balance])
    |> validate_format(:phone_number, ~r/^\+(\d{1,3})?\d{10}$/, message: "Invalid phone number. Should start with plus and from 10 to 13 digits")
    |> validate_format(:card_number, ~r/[0-9]{13,}/, message: "Card number must be at least 13 digits")
    |> validate_format(:zip_code, ~r/^\d{5}$/, message: "Zip code must be 5 digits")
    |> validate_number(:balance, greater_than: 0)
    |> unique_constraint(:phone_number, name: :customer_unique_phone_number_constraint)
  end

  defp validate_email(changeset) do
    changeset
    |> validate_required([:email])
    |> validate_format(:email, ~r/^[^\s]+@[^\s]+$/, message: "must have the @ sign and no spaces")
    |> validate_length(:email, max: 160)
    |> unsafe_validate_unique(:email, Volt.Repo)
    |> unique_constraint(:email)
  end

  defp validate_password(changeset, opts) do
    changeset
    |> validate_required([:password])
    |> validate_length(:password, min: 12, max: 72)
    # |> validate_format(:password, ~r/[a-z]/, message: "at least one lower case character")
    # |> validate_format(:password, ~r/[A-Z]/, message: "at least one upper case character")
    # |> validate_format(:password, ~r/[!?@#$%^&*_0-9]/, message: "at least one digit or punctuation character")
    |> maybe_hash_password(opts)
  end

  defp maybe_hash_password(changeset, opts) do
    hash_password? = Keyword.get(opts, :hash_password, true)
    password = get_change(changeset, :password)

    if hash_password? && password && changeset.valid? do
      changeset
      |> put_change(:hashed_password, Pbkdf2.hash_pwd_salt(password))
      |> delete_change(:password)
    else
      changeset
    end
  end

  @doc """
  A customer changeset for changing the email.

  It requires the email to change otherwise an error is added.
  """
  def email_changeset(customer, attrs) do
    customer
    |> cast(attrs, [:email])
    |> validate_email()
    |> case do
      %{changes: %{email: _}} = changeset -> changeset
      %{} = changeset -> add_error(changeset, :email, "did not change")
    end
  end

  @doc """
  A customer changeset for changing the password.

  ## Options

    * `:hash_password` - Hashes the password so it can be stored securely
      in the database and ensures the password field is cleared to prevent
      leaks in the logs. If password hashing is not needed and clearing the
      password field is not desired (like when using this changeset for
      validations on a LiveView form), this option can be set to `false`.
      Defaults to `true`.
  """
  def password_changeset(customer, attrs, opts \\ []) do
    customer
    |> cast(attrs, [:password])
    |> validate_confirmation(:password, message: "does not match password")
    |> validate_password(opts)
  end

  def location_changeset(customer, attrs) do
    customer
    |> cast(attrs, [:address, :zip_code, :city])
    |> validate_required([:address, :zip_code, :city])
  end

  @doc """
  A customer changeset for changing the profile.
  """
  def profile_changeset(customer, attrs) do
    customer
    |> cast(attrs, [:first_name, :last_name, :phone_number, :birth_date, :address, :zip_code, :city, :card_number, :balance])
    |> validate_profile()
  end

  @doc """
  Confirms the account by setting `confirmed_at`.
  """
  def confirm_changeset(customer) do
    now = NaiveDateTime.utc_now() |> NaiveDateTime.truncate(:second)
    change(customer, confirmed_at: now)
  end

  @doc """
  Verifies the password.

  If there is no customer or the customer doesn't have a password, we call
  `Pbkdf2.no_user_verify/0` to avoid timing attacks.
  """
  def valid_password?(%Volt.Accounts.Customer{hashed_password: hashed_password}, password)
      when is_binary(hashed_password) and byte_size(password) > 0 do
    Pbkdf2.verify_pass(password, hashed_password)
  end

  def valid_password?(_, _) do
    Pbkdf2.no_user_verify()
    false
  end

  @doc """
  Validates the current password otherwise adds an error to the changeset.
  """
  def validate_current_password(changeset, password) do
    if valid_password?(changeset.data, password) do
      changeset
    else
      add_error(changeset, :current_password, "is not valid")
    end
  end
end
