defmodule VoltWeb.CustomerDashboardController do
  use VoltWeb, :controller
  import Volt.Geolocation
  import Ecto.Query, only: [from: 2]

  alias Volt.Sales.Item
  alias Volt.Sales.Order
  alias Volt.Sales.Basket
  alias Volt.Accounts.Restaurant
  alias Volt.Repo
  alias Volt.Accounts

  require Logger


  defmodule Restaurant_summary do
    defstruct [:id, :name, :dist, :price_level, :opening_time, :closing_time, :long, :lat, :isOpen, :display_address, :discount]
  end

  defp customer_restaurants(conn) do
    usr = conn.assigns.current_customer
    current_user_address = "&addressLine=" <> usr.address <> "&postalCode=" <> usr.zip_code <> "&locality=" <> usr.city
    restaurants =
      from(rest in Restaurant, join: i in assoc(rest, :items), group_by: rest.id, select: {rest.id, rest.name, rest.address, rest.zip_code, rest.city, rest.price_level, rest.opening_time, rest.closing_time, sum(i.discount)})
      |> Repo.all()
      |> Enum.map(fn {id, name, address, zip_code, city, price_level, opening_time, closing_time, discount} ->
          display_address = address <> ", " <> city <> " " <> zip_code
          rest_address = "&addressLine=" <> address <> "&postalCode=" <> zip_code <> "&locality=" <> city
          location = find_location(rest_address)
          [dist, _] = distance(rest_address, current_user_address)
          isOpen = restaurantOpen(opening_time, closing_time)
          %{id: id, name: name, dist: dist, price_level: price_level, lat: List.first(location), long: List.last(location), opening_time: opening_time, closing_time: closing_time, isOpen: isOpen, display_address: display_address, discount: discount}
        end)
      |> Enum.sort_by(fn(rest) -> {rest.dist} end)
      |> Enum.map(fn %{id: id, name: name, dist: dist, price_level: price_level, lat: lat, long: long, opening_time: opening_time, closing_time: closing_time, isOpen: isOpen, display_address: display_address, discount: discount} ->
          %Restaurant_summary{id: id, name: name, dist: dist, price_level: price_level, lat: lat, long: long, opening_time: opening_time, closing_time: closing_time, isOpen: isOpen, display_address: display_address, discount: discount}
        end)
    restaurants
  end

  defp restaurantOpen(opening_time, closing_time) do
    time_now = Time.utc_now()
    opening_time < time_now && time_now < closing_time
  end

  defp customer_current_location(conn) do
    usr = conn.assigns.current_customer
    current_user_address = "&addressLine=" <> usr.address <> "&postalCode=" <> usr.zip_code <> "&locality=" <> usr.city
    find_location(current_user_address)
  end

  def index(conn, params) do
    render(conn, "index.html", restaurants: customer_restaurants(conn), current_location: customer_current_location(conn), bing_map: get_map(), location_changeset: Volt.Accounts.Customer.location_changeset(conn.assigns.current_customer, params))
  end

  def update(conn, params) do
    %{"customer" => customer_params} = params
    customer = conn.assigns.current_customer

    Accounts.update_customer_location(customer, customer_params)

    conn
    |> redirect(to: Routes.customer_dashboard_path(conn, :index))
  end

  def add_to_basket(conn, %{"id" => id, "basket" => basket}) do

    item =
      from(i in Item, where: i.id == ^id and i.orderable == ^true)
      |> Repo.one()
      |> Repo.preload(:restaurant)

    restaurant =
      from(i in Restaurant, where: i.id == ^item.restaurant_id)
      |> Repo.one()

    customer = conn.assigns.current_customer

    # Parse item price, quantity and then calculate total price of item
    item_price =
      if item.discount > 0 do
        Float.floor(elem(Float.parse(item.unit_price),0) * (1-(item.discount/100)),2)
      else
        Float.parse(item.unit_price) |> elem(0)
      end

    item_quantity = Integer.parse(basket["quantity"]) |> elem(0)
    items_price = item_price * item_quantity

    basket = %Basket{quantity: item_quantity}

    basket = Ecto.build_assoc(item, :basket, basket)

    # Find order
    order_query = from order in Order,
                    where: order.customer_id == ^customer.id and order.restaurant_id == ^item.restaurant_id and order.order_status == "BASKET"

    # If order doesn't exist, create it, otherwise get existing one
    order_query_result = case Repo.one(order_query) do
      nil ->
        order = %Order{delivery_price: 2.5, delivery_time: nil, order_price: 0.0, order_status: "BASKET", preparation_time: nil, restaurant_message: nil, total_price: 2.5}
        order = Ecto.build_assoc(restaurant, :orders, order)
        order = Ecto.build_assoc(customer, :orders, order) |> Repo.insert!()
      order -> order
    end


    basket_query = from basket in Basket, where: basket.item_id == ^item.id and basket.order_id == ^order_query_result.id, limit: 1

    case Repo.one(basket_query) do
      # If basket doesn't exist, create one and change total order sum
      nil ->
        basket_struct = Ecto.build_assoc(order_query_result, :basket, basket)
        changeset = Basket.changeset(basket_struct, %{})

        new_items_total =  order_query_result.order_price + items_price
        new_order_total =  new_items_total + order_query_result.delivery_price

        order_query_result |> Ecto.Changeset.change(%{order_price: new_items_total, total_price: new_order_total}) |> Repo.update!()

        case Repo.insert(changeset) do
          {:ok, _basket} ->
            conn
            |> put_flash(:info, "Basket updated!") |> redirect(to: Routes.customer_dashboard_path(conn, :show, order_query_result.restaurant_id))
          {:error, %Ecto.Changeset{} = _changeset} ->
            conn |> put_flash(:error, "Something went wrong!")
        end

      # If basket already exists, update chosen item quantity and order sum
      basket ->
        new_items_total =  order_query_result.order_price + items_price - (item_price * basket.quantity)
        new_order_total =  new_items_total + order_query_result.delivery_price

        order_query_result |> Ecto.Changeset.change(%{order_price: new_items_total, total_price: new_order_total}) |> Repo.update!()
        basket |> Ecto.Changeset.change(%{quantity: item_quantity}) |> Repo.update!()
        conn |> put_flash(:info, "Basket updated!") |> redirect(to: Routes.customer_dashboard_path(conn, :show, restaurant))
    end
    conn |> redirect(to: Routes.customer_dashboard_path(conn, :show, restaurant))
  end


  def delete(conn, %{"basket_id" => basket_id, "item_id" => item_id, "quantity" => quantity, "price" => price, "restaurant_id" => _restaurant_id}) do
    delete_from_basket = from basket in Basket, where: basket.id == ^basket_id

    basket = Repo.one!(delete_from_basket) |> Repo.delete!()
    item_price = Float.parse(price) |> elem(0)
    quantity = Integer.parse(quantity) |> elem(0)
    item_total = item_price * quantity

    order = Repo.get!(Order, basket.order_id)
    item = Repo.get!(Item, item_id)

    discount = case item.discount do
      0 -> 0
      discount_percent -> discount_percent / 100
    end

    item_total = item_total * (1 - discount)

    case order.order_price - item_total do
      0.0 -> Ecto.Changeset.change(order, total_price: 2.5, order_price: order.order_price - item_total, delivery_price: 2.5) |> Repo.update!()
      _x -> Ecto.Changeset.change(order, total_price: order.total_price - item_total, order_price: order.order_price - item_total, delivery_price: 2.5) |> Repo.update!()
    end
    conn |> redirect(to: NavigationHistory.last_path(conn))
  end

  def show(conn, %{"id" => id}) do
    restaurant =
      from(res in Restaurant, where: res.id == ^id)
      |> Repo.one()
      items =
        from(i in Item, where: i.restaurant_id == ^restaurant.id and i.orderable == ^true)
        |> Repo.all()
        |> Repo.preload(:restaurant)
    render(conn, "restaurant_menu.html", restaurant: restaurant, items: items)
  end

  def order(conn, %{"id" => id}) do
    item =
      from(i in Item, where: i.id == ^id  and i.orderable == ^true) |> Repo.one() |> Repo.preload(:restaurant)

    # Default item quantity displayed to the user will be 1
    basket = %{quantity: 1}

    basket_struct = Ecto.build_assoc(item, :basket, basket)

    changeset = Basket.changeset(basket_struct, %{})

    render(conn, "item_ordering.html", item: item, changeset: changeset)
  end

  def basket(conn, %{"id" => id}) do
    customer = conn.assigns.current_customer

    order_query = from order in Order,
                    where: order.customer_id == ^customer.id and order.restaurant_id == ^id and order.order_status == "BASKET"


    case Repo.one(order_query) do
      nil -> render(conn, "basket.html", items_in_basket: %{}, items_total: 0.0, total_price: 0.0, delivery_price: 0.0, restaurant_id: id)
      order ->
        items_in_basket_query = from item in Item,
                             join: basket in Basket, on: basket.item_id == item.id,
                             where: item.restaurant_id == ^id and basket.order_id == ^order.id and item.orderable == ^true,
                             select: %{item_id: item.id, name: item.name, description: item.description, price: item.unit_price, quantity: basket.quantity, restaurant_id: item.restaurant_id, basket_id: basket.id, discount: item.discount}

        items_in_basket = Repo.all(items_in_basket_query)

        render(conn, "basket.html", items_in_basket: items_in_basket, items_total: order.order_price, total_price: order.total_price, delivery_price: order.delivery_price, restaurant_id: order.restaurant_id)
    end
  end

  def submit(conn, %{"id" => restaurant_id}) do
    customer = conn.assigns.current_customer
    order_query = from order in Order,
                    where: order.customer_id == ^customer.id and order.restaurant_id == ^restaurant_id and order.order_status == "BASKET"
    case _order = Repo.one(order_query) do
      nil -> conn |> put_flash(:error, "Maybe go buy something first, silly!:)") |> redirect(to: NavigationHistory.last_path(conn))
      order ->
        case order.order_price do
          0.0 -> conn |> put_flash(:error, "Maybe go buy something first, silly!:)") |> redirect(to: NavigationHistory.last_path(conn))
          _x ->
            balance = customer.balance
            case balance do
              balance when balance >= order.total_price ->
                order = Ecto.Changeset.change(order, order_status: "PENDING") |> Repo.update!()
                updated_balance = balance - order.total_price
                Accounts.update_customer_balance(customer, %{balance: updated_balance})
                conn |> put_flash(:info, "Your order has been submitted!") |> redirect(to: NavigationHistory.last_path(conn))
              balance when balance < order.total_price ->
                conn |> put_flash(:error, "Where's the money Lebowski?!>:(") |> redirect(to: NavigationHistory.last_path(conn))
            end
        end
    end
  end

  defp get_map(), do: "https://www.bing.com/api/maps/mapcontrol?callback=GetMap&key=" <> System.get_env("BING_MAP_KEY")

end
