defmodule VoltWeb.RestaurantDashboardController do
  use VoltWeb, :controller
  import Ecto.Query, only: [from: 2]
  import Ecto.Changeset
  import Volt.Geolocation
  alias Volt.Sales.{Order, Basket}
  alias Volt.{Repo, Accounts}
  alias Volt.Accounts.{Courier, Customer}
  alias VoltWeb.RestaurantAuth
  alias Volt.Sales
  require Logger

  plug :assign_email_and_password_changesets

  defp restaurant_pending_orders(restaurant) do
    orders =
      from(o in Order, where: o.order_status == ^"PENDING" and o.restaurant_id == ^restaurant.id)
      |> Repo.all()
      |> Repo.preload([:restaurant, :customer])
    orders
  end

  defp restaurant_in_process_orders(restaurant) do
    orders =
      from(o in Order, where: o.order_status == ^"IN_PROCESS" and o.restaurant_id == ^restaurant.id)
      |> Repo.all()
      |> Repo.preload([:restaurant, :customer])
    orders
  end

  defp restaurant_prepared_orders(restaurant) do
    orders =
      from(o in Order, where: o.order_status == ^"PREPARED" and o.restaurant_id == ^restaurant.id)
      |> Repo.all()
      |> Repo.preload([:restaurant, :customer])
    orders
  end

  def index(conn, _params) do
    rest = conn.assigns.current_restaurant
    render(conn, "index.html", pending: restaurant_pending_orders(rest), in_process: restaurant_in_process_orders(rest))
  end

  def history(conn, _params) do
    rest = conn.assigns.current_restaurant
    render(conn, "order_history.html", prepared: restaurant_prepared_orders(rest))
  end

  def prepared_info(conn, %{"id" => id}) do
    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    inside_orders =
      from(b in Basket, where: b.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    courier =
      if(!is_nil(order.courier_id)) do
        from(c in Courier, where: c.id == ^order.courier_id)
        |> Repo.one()
      else
        nil
      end

    render(conn, "order_prepared.html", order: order, inside_orders: inside_orders, courier: courier)
  end

  def update(conn, %{"action" => "update_email"} = params) do
    %{"current_password" => password, "restaurant" => restaurant_params} = params
    restaurant = conn.assigns.current_restaurant

    case Accounts.apply_restaurant_email(restaurant, password, restaurant_params) do
      {:ok, applied_restaurant} ->
        Accounts.deliver_restaurant_update_email_instructions(
          applied_restaurant,
          restaurant.email,
          &Routes.restaurant_settings_url(conn, :confirm_email, &1)
        )

        conn
        |> put_flash(
          :info,
          "A link to confirm your email change has been sent to the new address."
        )
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))

      {:error, changeset} ->
        render(conn, "edit.html", email_changeset: changeset)
    end
  end

  def update(conn, %{"action" => "update_password"} = params) do
    %{"current_password" => password, "restaurant" => restaurant_params} = params
    restaurant = conn.assigns.current_restaurant

    case Accounts.update_restaurant_password(restaurant, password, restaurant_params) do
      {:ok, restaurant} ->
        conn
        |> put_flash(:info, "Password updated successfully.")
        |> put_session(:restaurant_return_to, Routes.restaurant_settings_path(conn, :edit))
        |> RestaurantAuth.log_in_restaurant(restaurant)

      {:error, changeset} ->
        render(conn, "edit.html", password_changeset: changeset)
    end
  end

  def confirm_email(conn, %{"token" => token}) do
    case Accounts.update_restaurant_email(conn.assigns.current_restaurant, token) do
      :ok ->
        conn
        |> put_flash(:info, "Email changed successfully.")
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))

      :error ->
        conn
        |> put_flash(:error, "Email change link is invalid or it has expired.")
        |> redirect(to: Routes.restaurant_settings_path(conn, :edit))
    end
  end

  defp assign_email_and_password_changesets(conn, _opts) do
    restaurant = conn.assigns.current_restaurant

    conn
    |> assign(:email_changeset, Accounts.change_restaurant_email(restaurant))
    |> assign(:password_changeset, Accounts.change_restaurant_password(restaurant))
  end

  def show_accept(conn, %{"id" => id}) do
    order =
      from(o in Order, where: o.id == ^id and o.order_status == ^"PENDING")
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    order = %{order | preparation_time: NaiveDateTime.local_now()}

    changeset = Sales.change_order(order)
    render(conn, "order_acceptance.html", changeset: changeset, order: order, inside_orders: inside_orders)
  end

  defp calculate_delivery_time(order) do
    current_restaurant_address = "&addressLine=" <> order.restaurant.address <> "&postalCode=" <> order.restaurant.zip_code <> "&locality=" <> order.restaurant.city
    cust_address = "&addressLine=" <> order.customer.address <> "&postalCode=" <> order.customer.zip_code <> "&locality=" <> order.customer.city
    [_dist, time] = distance(current_restaurant_address, cust_address)
    ceil(time)
  end

  def accept(conn, %{"id" => id, "order" => %{"preparation_duration" => ""}}) do
    order =
      from(o in Order, where: o.id == ^id and o.order_status == ^"PENDING")
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    nouv = %{order | preparation_duration: nil}

    changeset = add_error(%{Sales.change_order(order, Map.from_struct(nouv)) | action: :update}, :preparation_duration, "You must provide a preparation duration estimation (minute)!")

    render(conn, "order_acceptance.html", changeset: changeset, order: order, inside_orders: inside_orders)
  end

  def accept(conn, %{"id" => id, "order" => %{"preparation_duration" => prep_duration}}) do
    order =
      from(o in Order, where: o.id == ^id and o.order_status == ^"PENDING")
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    prep_duration = String.to_integer(prep_duration)
    del_duration = calculate_delivery_time(order)
    prep_time = NaiveDateTime.add(NaiveDateTime.local_now(), prep_duration*60, :second)
    del_time = NaiveDateTime.add(prep_time, del_duration*60, :second)

    nouv = %{order | order_status: "IN_PROCESS", preparation_duration: prep_duration, preparation_time: prep_time, delivery_duration: del_duration, delivery_time: del_time}
    Logger.info(%{order: nouv})

    case Sales.update_order(order, Map.from_struct(nouv)) do
      {:ok, _order} ->
        conn
        |> put_flash(:info, "Order accepted successfully.")
        |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "order_acceptance.html", changeset: changeset, order: order, inside_orders: inside_orders)
    end
  end

  def prepared(conn, %{"id" => id}) do
    order =
      from(o in Order, where: o.id == ^id and o.order_status == ^"IN_PROCESS")
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    if(order.courier_id == nil) do
      conn
      |> put_flash(:error, "Order has to have a courier first!")
      |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
    else
      prep_time = NaiveDateTime.add(NaiveDateTime.local_now(), order.preparation_duration*60, :second)
      del_time = NaiveDateTime.add(prep_time, order.delivery_duration*60, :second)
      case Sales.update_order(order, Map.from_struct(%{order | order_status: "PREPARED", preparation_time: prep_time, delivery_time: del_time})) do
        {:ok, _order} ->
          conn
          |> put_flash(:info, "Order marked as prepared successfully.")
          |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
        {:error, %Ecto.Changeset{} = _changeset} ->
          conn
          |> put_flash(:error, "Couldn't mark order as prepared!")
          |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
      end
    end
  end

  def show_cancel(conn, %{"id" => id}) do
    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    changeset = Sales.change_order(order)
    render(conn, "order_cancel.html", changeset: changeset, order: order, inside_orders: inside_orders)
  end

  def cancel(conn, %{"id" => id, "order" => %{"restaurant_message" => ""}}) do
    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer, :courier])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    nouv = %{order | restaurant_message: ""}

    changeset = add_error(%{Sales.change_order(order, Map.from_struct(nouv)) | action: :update}, :restaurant_message, "You must provide a message")

    render(conn, "order_cancel.html", changeset: changeset, order: order, inside_orders: inside_orders)
  end

  def cancel(conn, %{"id" => id, "order" => %{"restaurant_message" => message}}) do
    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer, :courier])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    nouv = %{order | order_status: "CANCELLED", restaurant_message: message, preparation_time: nil, preparation_duration: nil, delivery_time: nil, delivery_duration: nil}

    case Sales.update_order(order, Map.from_struct(nouv)) do
      {:ok, _order} ->
        customer =
          from(c in Customer, where: c.id == ^order.customer_id)
          |> Repo.one()
        updated_balance = customer.balance+order.total_price
        case Accounts.update_customer_balance(customer, %{balance: updated_balance}) do
          {:ok, _} ->
            conn
            |> put_flash(:info, "Order canceled successfully.")
            |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
          {:error, changeset} ->
            render(conn, "order_cancel.html", changeset: changeset, order: order, inside_orders: inside_orders)
        end
      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "order_cancel.html", changeset: changeset, order: order, inside_orders: inside_orders)
    end
  end

  def show_manage(conn, %{"id" => id}) do
    order =
      from(o in Order, where: o.id == ^id)
      |> Repo.one()
      |> Repo.preload([:restaurant, :customer, :courier])

    inside_orders =
      from(i in Basket, where: i.order_id == ^id)
      |> Repo.all()
      |> Repo.preload([:items])

    changeset = Sales.change_order(order)
    render(conn, "manage_order.html", changeset: changeset, order: order, inside_orders: inside_orders)
  end

  # def manage(conn, %{"id" => id, "order" => %{"preparation_time" => prep}}) do
  #   order =
  #     from(o in Order, where: o.id == ^id)
  #     |> Repo.one()
  #     |> Repo.preload([:restaurant, :customer])

  #   inside_orders =
  #     from(i in Basket, where: i.order_id == ^id)
  #     |> Repo.all()
  #     |> Repo.preload([:items])

  #   nouv = %{order | preparation_time: prep}

  #   case Sales.update_order(order, Map.from_struct(nouv)) do
  #     {:ok, _order} ->
  #       conn
  #       |> put_flash(:info, "Preparation time edited successfully.")
  #       |> redirect(to: Routes.restaurant_dashboard_path(conn, :index))
  #     {:error, %Ecto.Changeset{} = changeset} ->
  #       render(conn, "manage_order.html", changeset: changeset, order: order, inside_orders: inside_orders)
  #   end
  # end
end
