defmodule VoltWeb.ConnCase do
  @moduledoc """
  This module defines the test case to be used by
  tests that require setting up a connection.

  Such tests rely on `Phoenix.ConnTest` and also
  import other functionality to make it easier
  to build common data structures and query the data layer.

  Finally, if the test case interacts with the database,
  we enable the SQL sandbox, so changes done to the database
  are reverted at the end of every test. If you are using
  PostgreSQL, you can even run database tests asynchronously
  by setting `use VoltWeb.ConnCase, async: true`, although
  this option is not recommended for other databases.
  """

  use ExUnit.CaseTemplate

  using do
    quote do
      # Import conveniences for testing with connections
      import Plug.Conn
      import Phoenix.ConnTest
      import VoltWeb.ConnCase

      alias VoltWeb.Router.Helpers, as: Routes

      # The default endpoint for testing
      @endpoint VoltWeb.Endpoint
    end
  end

  setup tags do
    Volt.DataCase.setup_sandbox(tags)
    {:ok, conn: Phoenix.ConnTest.build_conn()}
  end

  @doc """
  Setup helper that registers and logs in couriers.

      setup :register_and_log_in_courier

  It stores an updated connection and a registered courier in the
  test context.
  """
  def register_and_log_in_courier(%{conn: conn}) do
    courier = Volt.AccountsFixtures.courier_fixture()
    %{conn: log_in_courier(conn, courier), courier: courier}
  end

  @doc """
  Logs the given `courier` into the `conn`.

  It returns an updated `conn`.
  """
  def log_in_courier(conn, courier) do
    token = Volt.Accounts.generate_courier_session_token(courier)

    conn
    |> Phoenix.ConnTest.init_test_session(%{})
    |> Plug.Conn.put_session(:courier_token, token)
  end

  @doc """
  Setup helper that registers and logs in restaurants.

      setup :register_and_log_in_restaurant

  It stores an updated connection and a registered restaurant in the
  test context.
  """
  def register_and_log_in_restaurant(%{conn: conn}) do
    restaurant = Volt.AccountsFixtures.restaurant_fixture()
    %{conn: log_in_restaurant(conn, restaurant), restaurant: restaurant}
  end

  @doc """
  Logs the given `restaurant` into the `conn`.

  It returns an updated `conn`.
  """
  def log_in_restaurant(conn, restaurant) do
    token = Volt.Accounts.generate_restaurant_session_token(restaurant)

    conn
    |> Phoenix.ConnTest.init_test_session(%{})
    |> Plug.Conn.put_session(:restaurant_token, token)
  end

  @doc """
  Setup helper that registers and logs in customers.

      setup :register_and_log_in_customer

  It stores an updated connection and a registered customer in the
  test context.
  """
  def register_and_log_in_customer(%{conn: conn}) do
    customer = Volt.AccountsFixtures.customer_fixture()
    %{conn: log_in_customer(conn, customer), customer: customer}
  end

  @doc """
  Logs the given `customer` into the `conn`.

  It returns an updated `conn`.
  """
  def log_in_customer(conn, customer) do
    token = Volt.Accounts.generate_customer_session_token(customer)

    conn
    |> Phoenix.ConnTest.init_test_session(%{})
    |> Plug.Conn.put_session(:customer_token, token)
  end
end
