defmodule Volt.AccountsTest do
  use Volt.DataCase

  alias Volt.Accounts
  alias Volt.Repo


  import Volt.AccountsFixtures
  alias Volt.Accounts.{Courier, CourierToken}

  describe "get_courier_by_email/1" do
    test "does not return the courier if the email does not exist" do
      refute Accounts.get_courier_by_email("unknown@example.com")
    end

    test "returns the courier if the email exists" do
      %{id: id} = courier = courier_fixture()
      assert %Courier{id: ^id} = Accounts.get_courier_by_email(courier.email)
    end
  end

  describe "get_courier_by_email_and_password/2" do
    test "does not return the courier if the email does not exist" do
      refute Accounts.get_courier_by_email_and_password("unknown@example.com", "hello world!")
    end

    test "does not return the courier if the password is not valid" do
      courier = courier_fixture()
      refute Accounts.get_courier_by_email_and_password(courier.email, "invalid")
    end

    test "returns the courier if the email and password are valid" do
      %{id: id} = courier = courier_fixture()

      assert %Courier{id: ^id} =
               Accounts.get_courier_by_email_and_password(courier.email, valid_courier_password())
    end
  end

  describe "get_courier!/1" do
    test "raises if id is invalid" do
      assert_raise Ecto.NoResultsError, fn ->
        Accounts.get_courier!(-1)
      end
    end

    test "returns the courier with the given id" do
      %{id: id} = courier = courier_fixture()
      assert %Courier{id: ^id} = Accounts.get_courier!(courier.id)
    end
  end

  describe "register_courier/1" do
    test "requires email and password to be set" do
      {:error, changeset} = Accounts.register_courier(%{})

      assert %{
               password: ["can't be blank"],
               email: ["can't be blank"]
             } = errors_on(changeset)
    end

    test "validates email and password when given" do
      {:error, changeset} = Accounts.register_courier(%{email: "not valid", password: "not valid"})

      assert %{
               email: ["must have the @ sign and no spaces"],
               password: ["should be at least 12 character(s)"]
             } = errors_on(changeset)
    end

    test "validates maximum values for email and password for security" do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.register_courier(%{email: too_long, password: too_long})
      assert "should be at most 160 character(s)" in errors_on(changeset).email
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates email uniqueness" do
      %{email: email} = courier_fixture()
      {:error, changeset} = Accounts.register_courier(%{email: email})
      assert "has already been taken" in errors_on(changeset).email

      # Now try with the upper cased email too, to check that email case is ignored.
      {:error, changeset} = Accounts.register_courier(%{email: String.upcase(email)})
      assert "has already been taken" in errors_on(changeset).email
    end

    test "registers couriers with a hashed password" do
      email = unique_courier_email()
      {:ok, courier} = Accounts.register_courier(valid_courier_attributes(email: email))
      assert courier.email == email
      assert is_binary(courier.hashed_password)
      assert is_nil(courier.confirmed_at)
      assert is_nil(courier.password)
    end
  end

  describe "change_courier_registration/2" do
    test "returns a changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_courier_registration(%Courier{})
      assert changeset.required == [:password, :email, :first_name, :last_name, :phone_number]
    end

    test "allows fields to be set" do
      email = unique_courier_email()
      password = valid_courier_password()

      changeset =
        Accounts.change_courier_registration(
          %Courier{},
          valid_courier_attributes(email: email, password: password)
        )

      assert changeset.valid?
      assert get_change(changeset, :email) == email
      assert get_change(changeset, :password) == password
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "change_courier_email/2" do
    test "returns a courier changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_courier_email(%Courier{})
      assert changeset.required == [:email]
    end
  end

  describe "apply_courier_email/3" do
    setup do
      %{courier: courier_fixture()}
    end

    test "requires email to change", %{courier: courier} do
      {:error, changeset} = Accounts.apply_courier_email(courier, valid_courier_password(), %{})
      assert %{email: ["did not change"]} = errors_on(changeset)
    end

    test "validates email", %{courier: courier} do
      {:error, changeset} =
        Accounts.apply_courier_email(courier, valid_courier_password(), %{email: "not valid"})

      assert %{email: ["must have the @ sign and no spaces"]} = errors_on(changeset)
    end

    test "validates maximum value for email for security", %{courier: courier} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.apply_courier_email(courier, valid_courier_password(), %{email: too_long})

      assert "should be at most 160 character(s)" in errors_on(changeset).email
    end

    test "validates email uniqueness", %{courier: courier} do
      %{email: email} = courier_fixture()

      {:error, changeset} =
        Accounts.apply_courier_email(courier, valid_courier_password(), %{email: email})

      assert "has already been taken" in errors_on(changeset).email
    end

    test "validates current password", %{courier: courier} do
      {:error, changeset} =
        Accounts.apply_courier_email(courier, "invalid", %{email: unique_courier_email()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "applies the email without persisting it", %{courier: courier} do
      email = unique_courier_email()
      {:ok, courier} = Accounts.apply_courier_email(courier, valid_courier_password(), %{email: email})
      assert courier.email == email
      assert Accounts.get_courier!(courier.id).email != email
    end
  end

  describe "deliver_courier_update_email_instructions/3" do
    setup do
      %{courier: courier_fixture()}
    end

    test "sends token through notification", %{courier: courier} do
      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_update_email_instructions(courier, "current@example.com", url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert courier_token = Repo.get_by(CourierToken, token: :crypto.hash(:sha256, token))
      assert courier_token.courier_id == courier.id
      assert courier_token.sent_to == courier.email
      assert courier_token.context == "change:current@example.com"
    end
  end

  describe "update_courier_email/2" do
    setup do
      courier = courier_fixture()
      email = unique_courier_email()

      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_update_email_instructions(%{courier | email: email}, courier.email, url)
        end)

      %{courier: courier, token: token, email: email}
    end

    test "updates the email with a valid token", %{courier: courier, token: token, email: email} do
      assert Accounts.update_courier_email(courier, token) == :ok
      changed_courier = Repo.get!(Courier, courier.id)
      assert changed_courier.email != courier.email
      assert changed_courier.email == email
      assert changed_courier.confirmed_at
      assert changed_courier.confirmed_at != courier.confirmed_at
      refute Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not update email with invalid token", %{courier: courier} do
      assert Accounts.update_courier_email(courier, "oops") == :error
      assert Repo.get!(Courier, courier.id).email == courier.email
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not update email if courier email changed", %{courier: courier, token: token} do
      assert Accounts.update_courier_email(%{courier | email: "current@example.com"}, token) == :error
      assert Repo.get!(Courier, courier.id).email == courier.email
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not update email if token expired", %{courier: courier, token: token} do
      {1, nil} = Repo.update_all(CourierToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.update_courier_email(courier, token) == :error
      assert Repo.get!(Courier, courier.id).email == courier.email
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end
  end

  describe "change_courier_password/2" do
    test "returns a courier changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_courier_password(%Courier{})
      assert changeset.required == [:password]
    end

    test "allows fields to be set" do
      changeset =
        Accounts.change_courier_password(%Courier{}, %{
          "password" => "new valid password"
        })

      assert changeset.valid?
      assert get_change(changeset, :password) == "new valid password"
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "update_courier_password/3" do
    setup do
      %{courier: courier_fixture()}
    end

    test "validates password", %{courier: courier} do
      {:error, changeset} =
        Accounts.update_courier_password(courier, valid_courier_password(), %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{courier: courier} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.update_courier_password(courier, valid_courier_password(), %{password: too_long})

      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates current password", %{courier: courier} do
      {:error, changeset} =
        Accounts.update_courier_password(courier, "invalid", %{password: valid_courier_password()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "updates the password", %{courier: courier} do
      {:ok, courier} =
        Accounts.update_courier_password(courier, valid_courier_password(), %{
          password: "new valid password"
        })

      assert is_nil(courier.password)
      assert Accounts.get_courier_by_email_and_password(courier.email, "new valid password")
    end

    test "deletes all tokens for the given courier", %{courier: courier} do
      _ = Accounts.generate_courier_session_token(courier)

      {:ok, _} =
        Accounts.update_courier_password(courier, valid_courier_password(), %{
          password: "new valid password"
        })

      refute Repo.get_by(CourierToken, courier_id: courier.id)
    end
  end

  describe "generate_courier_session_token/1" do
    setup do
      %{courier: courier_fixture()}
    end

    test "generates a token", %{courier: courier} do
      token = Accounts.generate_courier_session_token(courier)
      assert courier_token = Repo.get_by(CourierToken, token: token)
      assert courier_token.context == "session"

      # Creating the same token for another courier should fail
      assert_raise Ecto.ConstraintError, fn ->
        Repo.insert!(%CourierToken{
          token: courier_token.token,
          courier_id: courier_fixture().id,
          context: "session"
        })
      end
    end
  end

  describe "get_courier_by_session_token/1" do
    setup do
      courier = courier_fixture()
      token = Accounts.generate_courier_session_token(courier)
      %{courier: courier, token: token}
    end

    test "returns courier by token", %{courier: courier, token: token} do
      assert session_courier = Accounts.get_courier_by_session_token(token)
      assert session_courier.id == courier.id
    end

    test "does not return courier for invalid token" do
      refute Accounts.get_courier_by_session_token("oops")
    end

    test "does not return courier for expired token", %{token: token} do
      {1, nil} = Repo.update_all(CourierToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_courier_by_session_token(token)
    end
  end

  describe "delete_courier_session_token/1" do
    test "deletes the token" do
      courier = courier_fixture()
      token = Accounts.generate_courier_session_token(courier)
      assert Accounts.delete_courier_session_token(token) == :ok
      refute Accounts.get_courier_by_session_token(token)
    end
  end

  describe "deliver_courier_confirmation_instructions/2" do
    setup do
      %{courier: courier_fixture()}
    end

    test "sends token through notification", %{courier: courier} do
      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_confirmation_instructions(courier, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert courier_token = Repo.get_by(CourierToken, token: :crypto.hash(:sha256, token))
      assert courier_token.courier_id == courier.id
      assert courier_token.sent_to == courier.email
      assert courier_token.context == "confirm"
    end
  end

  describe "confirm_courier/1" do
    setup do
      courier = courier_fixture()

      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_confirmation_instructions(courier, url)
        end)

      %{courier: courier, token: token}
    end

    test "confirms the email with a valid token", %{courier: courier, token: token} do
      assert {:ok, confirmed_courier} = Accounts.confirm_courier(token)
      assert confirmed_courier.confirmed_at
      assert confirmed_courier.confirmed_at != courier.confirmed_at
      assert Repo.get!(Courier, courier.id).confirmed_at
      refute Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not confirm with invalid token", %{courier: courier} do
      assert Accounts.confirm_courier("oops") == :error
      refute Repo.get!(Courier, courier.id).confirmed_at
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not confirm email if token expired", %{courier: courier, token: token} do
      {1, nil} = Repo.update_all(CourierToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.confirm_courier(token) == :error
      refute Repo.get!(Courier, courier.id).confirmed_at
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end
  end

  describe "deliver_courier_reset_password_instructions/2" do
    setup do
      %{courier: courier_fixture()}
    end

    test "sends token through notification", %{courier: courier} do
      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_reset_password_instructions(courier, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert courier_token = Repo.get_by(CourierToken, token: :crypto.hash(:sha256, token))
      assert courier_token.courier_id == courier.id
      assert courier_token.sent_to == courier.email
      assert courier_token.context == "reset_password"
    end
  end

  describe "get_courier_by_reset_password_token/1" do
    setup do
      courier = courier_fixture()

      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_reset_password_instructions(courier, url)
        end)

      %{courier: courier, token: token}
    end

    test "returns the courier with valid token", %{courier: %{id: id}, token: token} do
      assert %Courier{id: ^id} = Accounts.get_courier_by_reset_password_token(token)
      assert Repo.get_by(CourierToken, courier_id: id)
    end

    test "does not return the courier with invalid token", %{courier: courier} do
      refute Accounts.get_courier_by_reset_password_token("oops")
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end

    test "does not return the courier if token expired", %{courier: courier, token: token} do
      {1, nil} = Repo.update_all(CourierToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_courier_by_reset_password_token(token)
      assert Repo.get_by(CourierToken, courier_id: courier.id)
    end
  end

  describe "reset_courier_password/2" do
    setup do
      %{courier: courier_fixture()}
    end

    test "validates password", %{courier: courier} do
      {:error, changeset} =
        Accounts.reset_courier_password(courier, %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{courier: courier} do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.reset_courier_password(courier, %{password: too_long})
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "updates the password", %{courier: courier} do
      {:ok, updated_courier} = Accounts.reset_courier_password(courier, %{password: "new valid password"})
      assert is_nil(updated_courier.password)
      assert Accounts.get_courier_by_email_and_password(courier.email, "new valid password")
    end

    test "deletes all tokens for the given courier", %{courier: courier} do
      _ = Accounts.generate_courier_session_token(courier)
      {:ok, _} = Accounts.reset_courier_password(courier, %{password: "new valid password"})
      refute Repo.get_by(CourierToken, courier_id: courier.id)
    end
  end

  describe "courier_inspect/2" do
    test "does not include password" do
      refute inspect(%Courier{password: "123456"}) =~ "password: \"123456\""
    end
  end

  import Volt.AccountsFixtures
  alias Volt.Accounts.{Restaurant, RestaurantToken}

  describe "get_restaurant_by_email/1" do
    test "does not return the restaurant if the email does not exist" do
      refute Accounts.get_restaurant_by_email("unknown@example.com")
    end

    test "returns the restaurant if the email exists" do
      %{id: id} = restaurant = restaurant_fixture()
      assert %Restaurant{id: ^id} = Accounts.get_restaurant_by_email(restaurant.email)
    end
  end

  describe "get_restaurant_by_email_and_password/2" do
    test "does not return the restaurant if the email does not exist" do
      refute Accounts.get_restaurant_by_email_and_password("unknown@example.com", "hello world!")
    end

    test "does not return the restaurant if the password is not valid" do
      restaurant = restaurant_fixture()
      refute Accounts.get_restaurant_by_email_and_password(restaurant.email, "invalid")
    end

    test "returns the restaurant if the email and password are valid" do
      %{id: id} = restaurant = restaurant_fixture()

      assert %Restaurant{id: ^id} =
               Accounts.get_restaurant_by_email_and_password(restaurant.email, valid_restaurant_password())
    end
  end

  describe "get_restaurant!/1" do
    test "raises if id is invalid" do
      assert_raise Ecto.NoResultsError, fn ->
        Accounts.get_restaurant!(-1)
      end
    end

    test "returns the restaurant with the given id" do
      %{id: id} = restaurant = restaurant_fixture()
      assert %Restaurant{id: ^id} = Accounts.get_restaurant!(restaurant.id)
    end
  end

  describe "register_restaurant/1" do
    test "requires email and password to be set" do
      {:error, changeset} = Accounts.register_restaurant(%{})

      assert %{
               password: ["can't be blank"],
               email: ["can't be blank"]
             } = errors_on(changeset)
    end

    test "validates email and password when given" do
      {:error, changeset} = Accounts.register_restaurant(%{email: "not valid", password: "not valid"})

      assert %{
               email: ["must have the @ sign and no spaces"],
               password: ["should be at least 12 character(s)"]
             } = errors_on(changeset)
    end

    test "validates maximum values for email and password for security" do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.register_restaurant(%{email: too_long, password: too_long})
      assert "should be at most 160 character(s)" in errors_on(changeset).email
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates email uniqueness" do
      %{email: email} = restaurant_fixture()
      {:error, changeset} = Accounts.register_restaurant(%{email: email})
      assert "has already been taken" in errors_on(changeset).email

      # Now try with the upper cased email too, to check that email case is ignored.
      {:error, changeset} = Accounts.register_restaurant(%{email: String.upcase(email)})
      assert "has already been taken" in errors_on(changeset).email
    end

    test "registers restaurants with a hashed password" do
      email = unique_restaurant_email()
      {:ok, restaurant} = Accounts.register_restaurant(valid_restaurant_attributes(email: email))
      assert restaurant.email == email
      assert is_binary(restaurant.hashed_password)
      assert is_nil(restaurant.confirmed_at)
      assert is_nil(restaurant.password)
    end
  end

  describe "change_restaurant_registration/2" do
    test "returns a changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_restaurant_registration(%Restaurant{})
      assert changeset.required == [:password, :email, :first_name, :last_name, :phone_number, :name, :address, :price_level, :opening_time, :closing_time, :zip_code, :city]
    end

    test "allows fields to be set" do
      email = unique_restaurant_email()
      password = valid_restaurant_password()

      changeset =
        Accounts.change_restaurant_registration(
          %Restaurant{},
          valid_restaurant_attributes(email: email, password: password)
        )

      assert changeset.valid?
      assert get_change(changeset, :email) == email
      assert get_change(changeset, :password) == password
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "change_restaurant_email/2" do
    test "returns a restaurant changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_restaurant_email(%Restaurant{})
      assert changeset.required == [:email]
    end
  end

  describe "apply_restaurant_email/3" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "requires email to change", %{restaurant: restaurant} do
      {:error, changeset} = Accounts.apply_restaurant_email(restaurant, valid_restaurant_password(), %{})
      assert %{email: ["did not change"]} = errors_on(changeset)
    end

    test "validates email", %{restaurant: restaurant} do
      {:error, changeset} =
        Accounts.apply_restaurant_email(restaurant, valid_restaurant_password(), %{email: "not valid"})

      assert %{email: ["must have the @ sign and no spaces"]} = errors_on(changeset)
    end

    test "validates maximum value for email for security", %{restaurant: restaurant} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.apply_restaurant_email(restaurant, valid_restaurant_password(), %{email: too_long})

      assert "should be at most 160 character(s)" in errors_on(changeset).email
    end

    test "validates email uniqueness", %{restaurant: restaurant} do
      %{email: email} = restaurant_fixture()

      {:error, changeset} =
        Accounts.apply_restaurant_email(restaurant, valid_restaurant_password(), %{email: email})

      assert "has already been taken" in errors_on(changeset).email
    end

    test "validates current password", %{restaurant: restaurant} do
      {:error, changeset} =
        Accounts.apply_restaurant_email(restaurant, "invalid", %{email: unique_restaurant_email()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "applies the email without persisting it", %{restaurant: restaurant} do
      email = unique_restaurant_email()
      {:ok, restaurant} = Accounts.apply_restaurant_email(restaurant, valid_restaurant_password(), %{email: email})
      assert restaurant.email == email
      assert Accounts.get_restaurant!(restaurant.id).email != email
    end
  end

  describe "deliver_restaurant_update_email_instructions/3" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "sends token through notification", %{restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_update_email_instructions(restaurant, "current@example.com", url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert restaurant_token = Repo.get_by(RestaurantToken, token: :crypto.hash(:sha256, token))
      assert restaurant_token.restaurant_id == restaurant.id
      assert restaurant_token.sent_to == restaurant.email
      assert restaurant_token.context == "change:current@example.com"
    end
  end

  describe "update_restaurant_email/2" do
    setup do
      restaurant = restaurant_fixture()
      email = unique_restaurant_email()

      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_update_email_instructions(%{restaurant | email: email}, restaurant.email, url)
        end)

      %{restaurant: restaurant, token: token, email: email}
    end

    test "updates the email with a valid token", %{restaurant: restaurant, token: token, email: email} do
      assert Accounts.update_restaurant_email(restaurant, token) == :ok
      changed_restaurant = Repo.get!(Restaurant, restaurant.id)
      assert changed_restaurant.email != restaurant.email
      assert changed_restaurant.email == email
      assert changed_restaurant.confirmed_at
      assert changed_restaurant.confirmed_at != restaurant.confirmed_at
      refute Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not update email with invalid token", %{restaurant: restaurant} do
      assert Accounts.update_restaurant_email(restaurant, "oops") == :error
      assert Repo.get!(Restaurant, restaurant.id).email == restaurant.email
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not update email if restaurant email changed", %{restaurant: restaurant, token: token} do
      assert Accounts.update_restaurant_email(%{restaurant | email: "current@example.com"}, token) == :error
      assert Repo.get!(Restaurant, restaurant.id).email == restaurant.email
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not update email if token expired", %{restaurant: restaurant, token: token} do
      {1, nil} = Repo.update_all(RestaurantToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.update_restaurant_email(restaurant, token) == :error
      assert Repo.get!(Restaurant, restaurant.id).email == restaurant.email
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end
  end

  describe "change_restaurant_password/2" do
    test "returns a restaurant changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_restaurant_password(%Restaurant{})
      assert changeset.required == [:password]
    end

    test "allows fields to be set" do
      changeset =
        Accounts.change_restaurant_password(%Restaurant{}, %{
          "password" => "new valid password"
        })

      assert changeset.valid?
      assert get_change(changeset, :password) == "new valid password"
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "update_restaurant_password/3" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "validates password", %{restaurant: restaurant} do
      {:error, changeset} =
        Accounts.update_restaurant_password(restaurant, valid_restaurant_password(), %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{restaurant: restaurant} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.update_restaurant_password(restaurant, valid_restaurant_password(), %{password: too_long})

      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates current password", %{restaurant: restaurant} do
      {:error, changeset} =
        Accounts.update_restaurant_password(restaurant, "invalid", %{password: valid_restaurant_password()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "updates the password", %{restaurant: restaurant} do
      {:ok, restaurant} =
        Accounts.update_restaurant_password(restaurant, valid_restaurant_password(), %{
          password: "new valid password"
        })

      assert is_nil(restaurant.password)
      assert Accounts.get_restaurant_by_email_and_password(restaurant.email, "new valid password")
    end

    test "deletes all tokens for the given restaurant", %{restaurant: restaurant} do
      _ = Accounts.generate_restaurant_session_token(restaurant)

      {:ok, _} =
        Accounts.update_restaurant_password(restaurant, valid_restaurant_password(), %{
          password: "new valid password"
        })

      refute Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end
  end

  describe "generate_restaurant_session_token/1" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "generates a token", %{restaurant: restaurant} do
      token = Accounts.generate_restaurant_session_token(restaurant)
      assert restaurant_token = Repo.get_by(RestaurantToken, token: token)
      assert restaurant_token.context == "session"

      # Creating the same token for another restaurant should fail
      assert_raise Ecto.ConstraintError, fn ->
        Repo.insert!(%RestaurantToken{
          token: restaurant_token.token,
          restaurant_id: restaurant_fixture().id,
          context: "session"
        })
      end
    end
  end

  describe "get_restaurant_by_session_token/1" do
    setup do
      restaurant = restaurant_fixture()
      token = Accounts.generate_restaurant_session_token(restaurant)
      %{restaurant: restaurant, token: token}
    end

    test "returns restaurant by token", %{restaurant: restaurant, token: token} do
      assert session_restaurant = Accounts.get_restaurant_by_session_token(token)
      assert session_restaurant.id == restaurant.id
    end

    test "does not return restaurant for invalid token" do
      refute Accounts.get_restaurant_by_session_token("oops")
    end

    test "does not return restaurant for expired token", %{token: token} do
      {1, nil} = Repo.update_all(RestaurantToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_restaurant_by_session_token(token)
    end
  end

  describe "delete_restaurant_session_token/1" do
    test "deletes the token" do
      restaurant = restaurant_fixture()
      token = Accounts.generate_restaurant_session_token(restaurant)
      assert Accounts.delete_restaurant_session_token(token) == :ok
      refute Accounts.get_restaurant_by_session_token(token)
    end
  end

  describe "deliver_restaurant_confirmation_instructions/2" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "sends token through notification", %{restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_confirmation_instructions(restaurant, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert restaurant_token = Repo.get_by(RestaurantToken, token: :crypto.hash(:sha256, token))
      assert restaurant_token.restaurant_id == restaurant.id
      assert restaurant_token.sent_to == restaurant.email
      assert restaurant_token.context == "confirm"
    end
  end

  describe "confirm_restaurant/1" do
    setup do
      restaurant = restaurant_fixture()

      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_confirmation_instructions(restaurant, url)
        end)

      %{restaurant: restaurant, token: token}
    end

    test "confirms the email with a valid token", %{restaurant: restaurant, token: token} do
      assert {:ok, confirmed_restaurant} = Accounts.confirm_restaurant(token)
      assert confirmed_restaurant.confirmed_at
      assert confirmed_restaurant.confirmed_at != restaurant.confirmed_at
      assert Repo.get!(Restaurant, restaurant.id).confirmed_at
      refute Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not confirm with invalid token", %{restaurant: restaurant} do
      assert Accounts.confirm_restaurant("oops") == :error
      refute Repo.get!(Restaurant, restaurant.id).confirmed_at
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not confirm email if token expired", %{restaurant: restaurant, token: token} do
      {1, nil} = Repo.update_all(RestaurantToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.confirm_restaurant(token) == :error
      refute Repo.get!(Restaurant, restaurant.id).confirmed_at
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end
  end

  describe "deliver_restaurant_reset_password_instructions/2" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "sends token through notification", %{restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_reset_password_instructions(restaurant, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert restaurant_token = Repo.get_by(RestaurantToken, token: :crypto.hash(:sha256, token))
      assert restaurant_token.restaurant_id == restaurant.id
      assert restaurant_token.sent_to == restaurant.email
      assert restaurant_token.context == "reset_password"
    end
  end

  describe "get_restaurant_by_reset_password_token/1" do
    setup do
      restaurant = restaurant_fixture()

      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_reset_password_instructions(restaurant, url)
        end)

      %{restaurant: restaurant, token: token}
    end

    test "returns the restaurant with valid token", %{restaurant: %{id: id}, token: token} do
      assert %Restaurant{id: ^id} = Accounts.get_restaurant_by_reset_password_token(token)
      assert Repo.get_by(RestaurantToken, restaurant_id: id)
    end

    test "does not return the restaurant with invalid token", %{restaurant: restaurant} do
      refute Accounts.get_restaurant_by_reset_password_token("oops")
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not return the restaurant if token expired", %{restaurant: restaurant, token: token} do
      {1, nil} = Repo.update_all(RestaurantToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_restaurant_by_reset_password_token(token)
      assert Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end
  end

  describe "reset_restaurant_password/2" do
    setup do
      %{restaurant: restaurant_fixture()}
    end

    test "validates password", %{restaurant: restaurant} do
      {:error, changeset} =
        Accounts.reset_restaurant_password(restaurant, %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{restaurant: restaurant} do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.reset_restaurant_password(restaurant, %{password: too_long})
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "updates the password", %{restaurant: restaurant} do
      {:ok, updated_restaurant} = Accounts.reset_restaurant_password(restaurant, %{password: "new valid password"})
      assert is_nil(updated_restaurant.password)
      assert Accounts.get_restaurant_by_email_and_password(restaurant.email, "new valid password")
    end

    test "deletes all tokens for the given restaurant", %{restaurant: restaurant} do
      _ = Accounts.generate_restaurant_session_token(restaurant)
      {:ok, _} = Accounts.reset_restaurant_password(restaurant, %{password: "new valid password"})
      refute Repo.get_by(RestaurantToken, restaurant_id: restaurant.id)
    end
  end

  describe "restaurant_inspect/2" do
    test "does not include password" do
      refute inspect(%Restaurant{password: "123456"}) =~ "password: \"123456\""
    end
  end

  import Volt.AccountsFixtures
  alias Volt.Accounts.{Customer, CustomerToken}

  describe "get_customer_by_email/1" do
    test "does not return the customer if the email does not exist" do
      refute Accounts.get_customer_by_email("unknown@example.com")
    end

    test "returns the customer if the email exists" do
      %{id: id} = customer = customer_fixture()
      assert %Customer{id: ^id} = Accounts.get_customer_by_email(customer.email)
    end
  end

  describe "get_customer_by_email_and_password/2" do
    test "does not return the customer if the email does not exist" do
      refute Accounts.get_customer_by_email_and_password("unknown@example.com", "hello world!")
    end

    test "does not return the customer if the password is not valid" do
      customer = customer_fixture()
      refute Accounts.get_customer_by_email_and_password(customer.email, "invalid")
    end

    test "returns the customer if the email and password are valid" do
      %{id: id} = customer = customer_fixture()

      assert %Customer{id: ^id} =
               Accounts.get_customer_by_email_and_password(customer.email, valid_customer_password())
    end
  end

  describe "get_customer!/1" do
    test "raises if id is invalid" do
      assert_raise Ecto.NoResultsError, fn ->
        Accounts.get_customer!(-1)
      end
    end

    test "returns the customer with the given id" do
      %{id: id} = customer = customer_fixture()
      assert %Customer{id: ^id} = Accounts.get_customer!(customer.id)
    end
  end

  describe "register_customer/1" do
    test "requires email and password to be set" do
      {:error, changeset} = Accounts.register_customer(%{})

      assert %{
               password: ["can't be blank"],
               email: ["can't be blank"]
             } = errors_on(changeset)
    end

    test "validates email and password when given" do
      {:error, changeset} = Accounts.register_customer(%{email: "not valid", password: "not valid"})

      assert %{
               email: ["must have the @ sign and no spaces"],
               password: ["should be at least 12 character(s)"]
             } = errors_on(changeset)
    end

    test "validates maximum values for email and password for security" do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.register_customer(%{email: too_long, password: too_long})
      assert "should be at most 160 character(s)" in errors_on(changeset).email
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates email uniqueness" do
      %{email: email} = customer_fixture()
      {:error, changeset} = Accounts.register_customer(%{email: email})
      assert "has already been taken" in errors_on(changeset).email

      # Now try with the upper cased email too, to check that email case is ignored.
      {:error, changeset} = Accounts.register_customer(%{email: String.upcase(email)})
      assert "has already been taken" in errors_on(changeset).email
    end

    test "registers customers with a hashed password" do
      email = unique_customer_email()
      {:ok, customer} = Accounts.register_customer(valid_customer_attributes(email: email))
      assert customer.email == email
      assert is_binary(customer.hashed_password)
      assert is_nil(customer.confirmed_at)
      assert is_nil(customer.password)
    end
  end

  describe "change_customer_registration/2" do
    test "returns a changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_customer_registration(%Customer{})
      assert changeset.required == [:password, :email, :first_name, :last_name, :phone_number, :birth_date, :address, :zip_code, :city, :card_number, :balance]
    end

    test "allows fields to be set" do
      email = unique_customer_email()
      password = valid_customer_password()

      changeset =
        Accounts.change_customer_registration(
          %Customer{},
          valid_customer_attributes(email: email, password: password)
        )

      assert changeset.valid?
      assert get_change(changeset, :email) == email
      assert get_change(changeset, :password) == password
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "change_customer_email/2" do
    test "returns a customer changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_customer_email(%Customer{})
      assert changeset.required == [:email]
    end
  end

  describe "apply_customer_email/3" do
    setup do
      %{customer: customer_fixture()}
    end

    test "requires email to change", %{customer: customer} do
      {:error, changeset} = Accounts.apply_customer_email(customer, valid_customer_password(), %{})
      assert %{email: ["did not change"]} = errors_on(changeset)
    end

    test "validates email", %{customer: customer} do
      {:error, changeset} =
        Accounts.apply_customer_email(customer, valid_customer_password(), %{email: "not valid"})

      assert %{email: ["must have the @ sign and no spaces"]} = errors_on(changeset)
    end

    test "validates maximum value for email for security", %{customer: customer} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.apply_customer_email(customer, valid_customer_password(), %{email: too_long})

      assert "should be at most 160 character(s)" in errors_on(changeset).email
    end

    test "validates email uniqueness", %{customer: customer} do
      %{email: email} = customer_fixture()

      {:error, changeset} =
        Accounts.apply_customer_email(customer, valid_customer_password(), %{email: email})

      assert "has already been taken" in errors_on(changeset).email
    end

    test "validates current password", %{customer: customer} do
      {:error, changeset} =
        Accounts.apply_customer_email(customer, "invalid", %{email: unique_customer_email()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "applies the email without persisting it", %{customer: customer} do
      email = unique_customer_email()
      {:ok, customer} = Accounts.apply_customer_email(customer, valid_customer_password(), %{email: email})
      assert customer.email == email
      assert Accounts.get_customer!(customer.id).email != email
    end
  end

  describe "deliver_customer_update_email_instructions/3" do
    setup do
      %{customer: customer_fixture()}
    end

    test "sends token through notification", %{customer: customer} do
      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_update_email_instructions(customer, "current@example.com", url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert customer_token = Repo.get_by(CustomerToken, token: :crypto.hash(:sha256, token))
      assert customer_token.customer_id == customer.id
      assert customer_token.sent_to == customer.email
      assert customer_token.context == "change:current@example.com"
    end
  end

  describe "update_customer_email/2" do
    setup do
      customer = customer_fixture()
      email = unique_customer_email()

      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_update_email_instructions(%{customer | email: email}, customer.email, url)
        end)

      %{customer: customer, token: token, email: email}
    end

    test "updates the email with a valid token", %{customer: customer, token: token, email: email} do
      assert Accounts.update_customer_email(customer, token) == :ok
      changed_customer = Repo.get!(Customer, customer.id)
      assert changed_customer.email != customer.email
      assert changed_customer.email == email
      assert changed_customer.confirmed_at
      assert changed_customer.confirmed_at != customer.confirmed_at
      refute Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not update email with invalid token", %{customer: customer} do
      assert Accounts.update_customer_email(customer, "oops") == :error
      assert Repo.get!(Customer, customer.id).email == customer.email
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not update email if customer email changed", %{customer: customer, token: token} do
      assert Accounts.update_customer_email(%{customer | email: "current@example.com"}, token) == :error
      assert Repo.get!(Customer, customer.id).email == customer.email
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not update email if token expired", %{customer: customer, token: token} do
      {1, nil} = Repo.update_all(CustomerToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.update_customer_email(customer, token) == :error
      assert Repo.get!(Customer, customer.id).email == customer.email
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end
  end

  describe "change_customer_password/2" do
    test "returns a customer changeset" do
      assert %Ecto.Changeset{} = changeset = Accounts.change_customer_password(%Customer{})
      assert changeset.required == [:password]
    end

    test "allows fields to be set" do
      changeset =
        Accounts.change_customer_password(%Customer{}, %{
          "password" => "new valid password"
        })

      assert changeset.valid?
      assert get_change(changeset, :password) == "new valid password"
      assert is_nil(get_change(changeset, :hashed_password))
    end
  end

  describe "update_customer_password/3" do
    setup do
      %{customer: customer_fixture()}
    end

    test "validates password", %{customer: customer} do
      {:error, changeset} =
        Accounts.update_customer_password(customer, valid_customer_password(), %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{customer: customer} do
      too_long = String.duplicate("db", 100)

      {:error, changeset} =
        Accounts.update_customer_password(customer, valid_customer_password(), %{password: too_long})

      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "validates current password", %{customer: customer} do
      {:error, changeset} =
        Accounts.update_customer_password(customer, "invalid", %{password: valid_customer_password()})

      assert %{current_password: ["is not valid"]} = errors_on(changeset)
    end

    test "updates the password", %{customer: customer} do
      {:ok, customer} =
        Accounts.update_customer_password(customer, valid_customer_password(), %{
          password: "new valid password"
        })

      assert is_nil(customer.password)
      assert Accounts.get_customer_by_email_and_password(customer.email, "new valid password")
    end

    test "deletes all tokens for the given customer", %{customer: customer} do
      _ = Accounts.generate_customer_session_token(customer)

      {:ok, _} =
        Accounts.update_customer_password(customer, valid_customer_password(), %{
          password: "new valid password"
        })

      refute Repo.get_by(CustomerToken, customer_id: customer.id)
    end
  end

  describe "generate_customer_session_token/1" do
    setup do
      %{customer: customer_fixture()}
    end

    test "generates a token", %{customer: customer} do
      token = Accounts.generate_customer_session_token(customer)
      assert customer_token = Repo.get_by(CustomerToken, token: token)
      assert customer_token.context == "session"

      # Creating the same token for another customer should fail
      assert_raise Ecto.ConstraintError, fn ->
        Repo.insert!(%CustomerToken{
          token: customer_token.token,
          customer_id: customer_fixture().id,
          context: "session"
        })
      end
    end
  end

  describe "get_customer_by_session_token/1" do
    setup do
      customer = customer_fixture()
      token = Accounts.generate_customer_session_token(customer)
      %{customer: customer, token: token}
    end

    test "returns customer by token", %{customer: customer, token: token} do
      assert session_customer = Accounts.get_customer_by_session_token(token)
      assert session_customer.id == customer.id
    end

    test "does not return customer for invalid token" do
      refute Accounts.get_customer_by_session_token("oops")
    end

    test "does not return customer for expired token", %{token: token} do
      {1, nil} = Repo.update_all(CustomerToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_customer_by_session_token(token)
    end
  end

  describe "delete_customer_session_token/1" do
    test "deletes the token" do
      customer = customer_fixture()
      token = Accounts.generate_customer_session_token(customer)
      assert Accounts.delete_customer_session_token(token) == :ok
      refute Accounts.get_customer_by_session_token(token)
    end
  end

  describe "deliver_customer_confirmation_instructions/2" do
    setup do
      %{customer: customer_fixture()}
    end

    test "sends token through notification", %{customer: customer} do
      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_confirmation_instructions(customer, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert customer_token = Repo.get_by(CustomerToken, token: :crypto.hash(:sha256, token))
      assert customer_token.customer_id == customer.id
      assert customer_token.sent_to == customer.email
      assert customer_token.context == "confirm"
    end
  end

  describe "confirm_customer/1" do
    setup do
      customer = customer_fixture()

      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_confirmation_instructions(customer, url)
        end)

      %{customer: customer, token: token}
    end

    test "confirms the email with a valid token", %{customer: customer, token: token} do
      assert {:ok, confirmed_customer} = Accounts.confirm_customer(token)
      assert confirmed_customer.confirmed_at
      assert confirmed_customer.confirmed_at != customer.confirmed_at
      assert Repo.get!(Customer, customer.id).confirmed_at
      refute Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not confirm with invalid token", %{customer: customer} do
      assert Accounts.confirm_customer("oops") == :error
      refute Repo.get!(Customer, customer.id).confirmed_at
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not confirm email if token expired", %{customer: customer, token: token} do
      {1, nil} = Repo.update_all(CustomerToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      assert Accounts.confirm_customer(token) == :error
      refute Repo.get!(Customer, customer.id).confirmed_at
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end
  end

  describe "deliver_customer_reset_password_instructions/2" do
    setup do
      %{customer: customer_fixture()}
    end

    test "sends token through notification", %{customer: customer} do
      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_reset_password_instructions(customer, url)
        end)

      {:ok, token} = Base.url_decode64(token, padding: false)
      assert customer_token = Repo.get_by(CustomerToken, token: :crypto.hash(:sha256, token))
      assert customer_token.customer_id == customer.id
      assert customer_token.sent_to == customer.email
      assert customer_token.context == "reset_password"
    end
  end

  describe "get_customer_by_reset_password_token/1" do
    setup do
      customer = customer_fixture()

      token =
        extract_customer_token(fn url ->
          Accounts.deliver_customer_reset_password_instructions(customer, url)
        end)

      %{customer: customer, token: token}
    end

    test "returns the customer with valid token", %{customer: %{id: id}, token: token} do
      assert %Customer{id: ^id} = Accounts.get_customer_by_reset_password_token(token)
      assert Repo.get_by(CustomerToken, customer_id: id)
    end

    test "does not return the customer with invalid token", %{customer: customer} do
      refute Accounts.get_customer_by_reset_password_token("oops")
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end

    test "does not return the customer if token expired", %{customer: customer, token: token} do
      {1, nil} = Repo.update_all(CustomerToken, set: [inserted_at: ~N[2020-01-01 00:00:00]])
      refute Accounts.get_customer_by_reset_password_token(token)
      assert Repo.get_by(CustomerToken, customer_id: customer.id)
    end
  end

  describe "reset_customer_password/2" do
    setup do
      %{customer: customer_fixture()}
    end

    test "validates password", %{customer: customer} do
      {:error, changeset} =
        Accounts.reset_customer_password(customer, %{
          password: "not valid",
          password_confirmation: "another"
        })

      assert %{
               password: ["should be at least 12 character(s)"],
               password_confirmation: ["does not match password"]
             } = errors_on(changeset)
    end

    test "validates maximum values for password for security", %{customer: customer} do
      too_long = String.duplicate("db", 100)
      {:error, changeset} = Accounts.reset_customer_password(customer, %{password: too_long})
      assert "should be at most 72 character(s)" in errors_on(changeset).password
    end

    test "updates the password", %{customer: customer} do
      {:ok, updated_customer} = Accounts.reset_customer_password(customer, %{password: "new valid password"})
      assert is_nil(updated_customer.password)
      assert Accounts.get_customer_by_email_and_password(customer.email, "new valid password")
    end

    test "deletes all tokens for the given customer", %{customer: customer} do
      _ = Accounts.generate_customer_session_token(customer)
      {:ok, _} = Accounts.reset_customer_password(customer, %{password: "new valid password"})
      refute Repo.get_by(CustomerToken, customer_id: customer.id)
    end
  end

  describe "customer_inspect/2" do
    test "does not include password" do
      refute inspect(%Customer{password: "123456"}) =~ "password: \"123456\""
    end
  end

"""
  TODO: Revise old tests
  describe "restaurants" do
    alias Volt.Accounts.Restaurant

    import Volt.AccountsFixtures

    @invalid_attrs %{address: nil, closing_time: nil, email: nil, first_name: nil, last_name: nil, name: nil, opening_time: nil, password: nil, phone_number: nil, price_level: nil}

    test "list_restaurants/0 returns all restaurants" do
      restaurant = restaurant_fixture()
      assert Accounts.list_restaurants() == [restaurant]
    end

    test "get_restaurant!/1 returns the restaurant with given id" do
      restaurant = restaurant_fixture()
      assert Accounts.get_restaurant!(restaurant.id) == restaurant
    end

    test "create_restaurant/1 with valid data creates a restaurant" do
      valid_attrs = %{address: "some address", closing_time: ~T[14:00:00], email: "some email", first_name: "some first_name", last_name: "some last_name", name: "some name", opening_time: ~T[14:00:00], password: "some password", phone_number: "some phone_number", price_level: 42}

      assert {:ok, %Restaurant{} = restaurant} = Accounts.create_restaurant(valid_attrs)
      assert restaurant.address == "some address"
      assert restaurant.closing_time == ~T[14:00:00]
      assert restaurant.email == "some@mail.com"
      assert restaurant.first_name == "some first_name"
      assert restaurant.last_name == "some last_name"
      assert restaurant.name == "Restaurant Name"
      assert restaurant.opening_time == ~T[14:00:00]
      assert restaurant.password == "some_password_123"
      assert restaurant.phone_number == "+37253584617"
      assert restaurant.price_level == 42
    end

    test "create_restaurant/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Accounts.create_restaurant(@invalid_attrs)
    end

    test "update_restaurant/2 with valid data updates the restaurant" do
      restaurant = restaurant_fixture()
      update_attrs = %{address: "some updated address", closing_time: ~T[15:01:01], email: "some updated email", first_name: "some updated first_name", last_name: "some updated last_name", name: "some updated name", opening_time: ~T[15:01:01], password: "some updated password", phone_number: "some updated phone_number", price_level: 43}

      assert {:ok, %Restaurant{} = restaurant} = Accounts.update_restaurant(restaurant, update_attrs)
      assert restaurant.address == "updated some address"
      assert restaurant.closing_time == ~T[14:00:00]
      assert restaurant.email == "some@mail.com"
      assert restaurant.first_name == "updated some first_name"
      assert restaurant.last_name == "updated some last_name"
      assert restaurant.name == "Updated Restaurant Name"
      assert restaurant.opening_time == ~T[14:00:00]
      assert restaurant.password == "updated_some_password_123"
      assert restaurant.phone_number == "+37253584617"
      assert restaurant.price_level == 42
    end

    test "update_restaurant/2 with invalid data returns error changeset" do
      restaurant = restaurant_fixture()
      assert {:error, %Ecto.Changeset{}} = Accounts.update_restaurant(restaurant, @invalid_attrs)
      assert restaurant == Accounts.get_restaurant!(restaurant.id)
    end

    test "delete_restaurant/1 deletes the restaurant" do
      restaurant = restaurant_fixture()
      assert {:ok, %Restaurant{}} = Accounts.delete_restaurant(restaurant)
      assert_raise Ecto.NoResultsError, fn -> Accounts.get_restaurant!(restaurant.id) end
    end

    test "change_restaurant/1 returns a restaurant changeset" do
      restaurant = restaurant_fixture()
      assert %Ecto.Changeset{} = Accounts.change_restaurant(restaurant)
    end
  end

  describe "couriers" do
    alias Volt.Accounts.Courier

    import Volt.AccountsFixtures

    @invalid_attrs %{courier_status: nil, email: nil, first_name: nil, last_name: nil, password: nil, phone_number: nil}

    test "list_couriers/0 returns all couriers" do
      courier = courier_fixture()
      assert Accounts.list_couriers() == [courier]
    end

    test "get_courier!/1 returns the courier with given id" do
      courier = courier_fixture()
      assert Accounts.get_courier!(courier.id) == courier
    end

    test "create_courier/1 with valid data creates a courier" do
      valid_attrs = %{courier_status: "some courier_status", email: "roro@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+33641941064"}

      assert {:ok, %Courier{} = courier} = Accounts.create_courier(valid_attrs)
      assert courier.courier_status == "some courier_status"
      assert courier.email == "roro@gmail.com"
      assert courier.first_name == "some first_name"
      assert courier.last_name == "some last_name"
      assert courier.password == "some password"
      assert courier.phone_number == "+33641941064"
    end

    test "create_courier/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Accounts.create_courier(@invalid_attrs)
    end

    test "update_courier/2 with valid data updates the courier" do
      courier = courier_fixture()
      update_attrs = %{courier_status: "some updated courier_status", email: "oagon@email.com", first_name: "some updated first_name", last_name: "some updated last_name", password: "some updated password", phone_number: "+37291827384"}

      assert {:ok, %Courier{} = courier} = Accounts.update_courier(courier, update_attrs)
      assert courier.courier_status == "some updated courier_status"
      assert courier.email == "oagon@email.com"
      assert courier.first_name == "some updated first_name"
      assert courier.last_name == "some updated last_name"
      assert courier.password == "some updated password"
      assert courier.phone_number == "+37291827384"
    end

    test "update_courier/2 with invalid data returns error changeset" do
      courier = courier_fixture()
      assert {:error, %Ecto.Changeset{}} = Accounts.update_courier(courier, @invalid_attrs)
      assert courier == Accounts.get_courier!(courier.id)
    end

    test "delete_courier/1 deletes the courier" do
      courier = courier_fixture()
      assert {:ok, %Courier{}} = Accounts.delete_courier(courier)
      assert_raise Ecto.NoResultsError, fn -> Accounts.get_courier!(courier.id) end
    end

    test "change_courier/1 returns a courier changeset" do
      courier = courier_fixture()
      assert %Ecto.Changeset{} = Accounts.change_courier(courier)
    end
  end

  describe "customers" do
    alias Volt.Accounts.Customer

    import Volt.AccountsFixtures

    @invalid_attrs %{address: nil, zip_code: "asd", city: nil, balance: nil, birth_date: nil, card_number: nil, email: "sdfzefz", first_name: nil, last_name: nil, password: "2qsd*", phone_number: "123455"}
    @valid_attrs %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "test@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+61212121267"}

    test "list_customers/0 returns all customers" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert Accounts.list_customers() == [customer]
    end

    test "get_customer!/1 returns the customer with given id" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert Accounts.get_customer!(customer.id) == customer
    end

    test "create_customer/1 with valid data creates a customer" do
      assert {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert customer.address == "some address"
      assert customer.zip_code == "50000"
      assert customer.city == "Tartu"
      assert customer.balance == 120.5
      assert customer.birth_date == ~D[2022-11-04]
      assert customer.card_number == "1234567891234"
      assert customer.email == "test@gmail.com"
      assert customer.first_name == "some first_name"
      assert customer.last_name == "some last_name"
      assert customer.password == "some password"
      assert customer.phone_number == "+61212121267"
    end

    test "create_customer/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Accounts.create_customer(@invalid_attrs)
    end

    test "update_customer/2 with valid data updates the customer" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      update_attrs = %{address: "some updated address", zip_code: "60000", city: "Parnu", balance: 456.7, birth_date: ~D[2022-11-05], card_number: "11211111111111111", email: "test@google.net", first_name: "some updated first_name", last_name: "some updated last_name", password: "some updated password", phone_number: "+33641941064"}

      assert {:ok, %Customer{} = customer} = Accounts.update_customer(customer, update_attrs)
      assert customer.address == "some updated address"
      assert customer.zip_code == "60000"
      assert customer.city == "Parnu"
      assert customer.balance == 456.7
      assert customer.birth_date == ~D[2022-11-05]
      assert customer.card_number == "11211111111111111"
      assert customer.email == "test@google.net"
      assert customer.first_name == "some updated first_name"
      assert customer.last_name == "some updated last_name"
      assert customer.password == "some updated password"
      assert customer.phone_number == "+33641941064"
    end

    test "update_customer/2 with invalid data returns error changeset" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert {:error, %Ecto.Changeset{}} = Accounts.update_customer(customer, @invalid_attrs)
      assert customer == Accounts.get_customer!(customer.id)
    end

    test "delete_customer/1 deletes the customer" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert {:ok, %Customer{}} = Accounts.delete_customer(customer)
      assert_raise Ecto.NoResultsError, fn -> Accounts.get_customer!(customer.id) end
    end

    test "change_customer/1 returns a customer changeset" do
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert %Ecto.Changeset{} = Accounts.change_customer(customer)
    end

    test "create_customer/1 with already taken email return error changeset" do
      new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "test@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "0413171819"}
      %Customer{}
        |> Customer.changeset(@valid_attrs)
        |> Repo.insert()
      c2 = %Customer{}
        |> Customer.changeset(new_attrs)
      assert {:error, changeset} = Repo.insert(c2)
    end

    test "update_customer/2 with already taken email returns error changeset" do
      update_new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "test@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+41317181987"}
      new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "1234@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+23123123489"}
      Accounts.create_customer(@valid_attrs)
      {:ok, %Customer{} = customer} = Accounts.create_customer(new_attrs)
      assert {:error, %Ecto.Changeset{}} = Accounts.update_customer(customer, update_new_attrs)
    end

    test "create_customer/1 with already taken phone_number return error changeset" do
      new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "new@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+61212121267"}
      {:ok, %Customer{} = customer} = Accounts.create_customer(@valid_attrs)
      assert {:error, %Ecto.Changeset{}} = Accounts.create_customer(new_attrs)
    end

    test "update_customer/2 with already taken phone_number returns error changeset" do
      update_new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "new@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+61212121267"}
      new_attrs = %{address: "some address", zip_code: "50000", city: "Tartu", balance: 120.5, birth_date: ~D[2022-11-04], card_number: "1234567891234", email: "1234@gmail.com", first_name: "some first_name", last_name: "some last_name", password: "some password", phone_number: "+23123123489"}
      Accounts.create_customer(@valid_attrs)
      {:ok, %Customer{} = customer} = Accounts.create_customer(new_attrs)
      assert {:error, %Ecto.Changeset{}} = Accounts.update_customer(customer, update_new_attrs)
    end
  end
  """
end
