defmodule VoltWeb.CourierAuthTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  alias VoltWeb.CourierAuth
  import Volt.AccountsFixtures

  @remember_me_cookie "_volt_web_courier_remember_me"

  setup %{conn: conn} do
    conn =
      conn
      |> Map.replace!(:secret_key_base, VoltWeb.Endpoint.config(:secret_key_base))
      |> init_test_session(%{})

    %{courier: courier_fixture(), conn: conn}
  end

  describe "log_in_courier/3" do
    test "stores the courier token in the session", %{conn: conn, courier: courier} do
      conn = CourierAuth.log_in_courier(conn, courier)
      assert token = get_session(conn, :courier_token)
      assert get_session(conn, :live_socket_id) == "couriers_sessions:#{Base.url_encode64(token)}"
      assert redirected_to(conn) == "/couriers/dashboard"
      assert Accounts.get_courier_by_session_token(token)
    end

    test "clears everything previously stored in the session", %{conn: conn, courier: courier} do
      conn = conn |> put_session(:to_be_removed, "value") |> CourierAuth.log_in_courier(courier)
      refute get_session(conn, :to_be_removed)
    end

    test "redirects to the configured path", %{conn: conn, courier: courier} do
      conn = conn |> put_session(:courier_return_to, "/hello") |> CourierAuth.log_in_courier(courier)
      assert redirected_to(conn) == "/hello"
    end

    test "writes a cookie if remember_me is configured", %{conn: conn, courier: courier} do
      conn = conn |> fetch_cookies() |> CourierAuth.log_in_courier(courier, %{"remember_me" => "true"})
      assert get_session(conn, :courier_token) == conn.cookies[@remember_me_cookie]

      assert %{value: signed_token, max_age: max_age} = conn.resp_cookies[@remember_me_cookie]
      assert signed_token != get_session(conn, :courier_token)
      assert max_age == 5_184_000
    end
  end

  describe "logout_courier/1" do
    test "erases session and cookies", %{conn: conn, courier: courier} do
      courier_token = Accounts.generate_courier_session_token(courier)

      conn =
        conn
        |> put_session(:courier_token, courier_token)
        |> put_req_cookie(@remember_me_cookie, courier_token)
        |> fetch_cookies()
        |> CourierAuth.log_out_courier()

      refute get_session(conn, :courier_token)
      refute conn.cookies[@remember_me_cookie]
      assert %{max_age: 0} = conn.resp_cookies[@remember_me_cookie]
      assert redirected_to(conn) == "/"
      refute Accounts.get_courier_by_session_token(courier_token)
    end

    test "broadcasts to the given live_socket_id", %{conn: conn} do
      live_socket_id = "couriers_sessions:abcdef-token"
      VoltWeb.Endpoint.subscribe(live_socket_id)

      conn
      |> put_session(:live_socket_id, live_socket_id)
      |> CourierAuth.log_out_courier()

      assert_receive %Phoenix.Socket.Broadcast{event: "disconnect", topic: ^live_socket_id}
    end

    test "works even if courier is already logged out", %{conn: conn} do
      conn = conn |> fetch_cookies() |> CourierAuth.log_out_courier()
      refute get_session(conn, :courier_token)
      assert %{max_age: 0} = conn.resp_cookies[@remember_me_cookie]
      assert redirected_to(conn) == "/"
    end
  end

  describe "fetch_current_courier/2" do
    test "authenticates courier from session", %{conn: conn, courier: courier} do
      courier_token = Accounts.generate_courier_session_token(courier)
      conn = conn |> put_session(:courier_token, courier_token) |> CourierAuth.fetch_current_courier([])
      assert conn.assigns.current_courier.id == courier.id
    end

    test "authenticates courier from cookies", %{conn: conn, courier: courier} do
      logged_in_conn =
        conn |> fetch_cookies() |> CourierAuth.log_in_courier(courier, %{"remember_me" => "true"})

      courier_token = logged_in_conn.cookies[@remember_me_cookie]
      %{value: signed_token} = logged_in_conn.resp_cookies[@remember_me_cookie]

      conn =
        conn
        |> put_req_cookie(@remember_me_cookie, signed_token)
        |> CourierAuth.fetch_current_courier([])

      assert get_session(conn, :courier_token) == courier_token
      assert conn.assigns.current_courier.id == courier.id
    end

    test "does not authenticate if data is missing", %{conn: conn, courier: courier} do
      _ = Accounts.generate_courier_session_token(courier)
      conn = CourierAuth.fetch_current_courier(conn, [])
      refute get_session(conn, :courier_token)
      refute conn.assigns.current_courier
    end
  end

  describe "redirect_if_courier_is_authenticated/2" do
    test "redirects if courier is authenticated", %{conn: conn, courier: courier} do
      conn = conn |> assign(:current_courier, courier) |> CourierAuth.redirect_if_courier_is_authenticated([])
      assert conn.halted
      assert redirected_to(conn) == "/couriers/dashboard"
    end

    test "does not redirect if courier is not authenticated", %{conn: conn} do
      conn = CourierAuth.redirect_if_courier_is_authenticated(conn, [])
      refute conn.halted
      refute conn.status
    end
  end

  describe "require_authenticated_courier/2" do
    test "redirects if courier is not authenticated", %{conn: conn} do
      conn = conn |> fetch_flash() |> CourierAuth.require_authenticated_courier([])
      assert conn.halted
      assert redirected_to(conn) == Routes.courier_session_path(conn, :new)
      assert get_flash(conn, :error) == "You must log in to access this page."
    end

    test "stores the path to redirect to on GET", %{conn: conn} do
      halted_conn =
        %{conn | path_info: ["foo"], query_string: ""}
        |> fetch_flash()
        |> CourierAuth.require_authenticated_courier([])

      assert halted_conn.halted
      assert get_session(halted_conn, :courier_return_to) == "/foo"

      halted_conn =
        %{conn | path_info: ["foo"], query_string: "bar=baz"}
        |> fetch_flash()
        |> CourierAuth.require_authenticated_courier([])

      assert halted_conn.halted
      assert get_session(halted_conn, :courier_return_to) == "/foo?bar=baz"

      halted_conn =
        %{conn | path_info: ["foo"], query_string: "bar", method: "POST"}
        |> fetch_flash()
        |> CourierAuth.require_authenticated_courier([])

      assert halted_conn.halted
      refute get_session(halted_conn, :courier_return_to)
    end

    test "does not redirect if courier is authenticated", %{conn: conn, courier: courier} do
      conn = conn |> assign(:current_courier, courier) |> CourierAuth.require_authenticated_courier([])
      refute conn.halted
      refute conn.status
    end
  end
end
