defmodule VoltWeb.CourierConfirmationControllerTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  alias Volt.Repo
  import Volt.AccountsFixtures

  setup do
    %{courier: courier_fixture()}
  end

  describe "GET /couriers/confirm" do
    test "renders the resend confirmation page", %{conn: conn} do
      conn = get(conn, Routes.courier_confirmation_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Resend confirmation instructions</h1>"
    end
  end

  describe "POST /couriers/confirm" do
    @tag :capture_log
    test "sends a new confirmation token", %{conn: conn, courier: courier} do
      conn =
        post(conn, Routes.courier_confirmation_path(conn, :create), %{
          "courier" => %{"email" => courier.email}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.get_by!(Accounts.CourierToken, courier_id: courier.id).context == "confirm"
    end

    test "does not send confirmation token if Courier is confirmed", %{conn: conn, courier: courier} do
      Repo.update!(Accounts.Courier.confirm_changeset(courier))

      conn =
        post(conn, Routes.courier_confirmation_path(conn, :create), %{
          "courier" => %{"email" => courier.email}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      refute Repo.get_by(Accounts.CourierToken, courier_id: courier.id)
    end

    test "does not send confirmation token if email is invalid", %{conn: conn} do
      conn =
        post(conn, Routes.courier_confirmation_path(conn, :create), %{
          "courier" => %{"email" => "unknown@example.com"}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.all(Accounts.CourierToken) == []
    end
  end

  describe "GET /couriers/confirm/:token" do
    test "renders the confirmation page", %{conn: conn} do
      conn = get(conn, Routes.courier_confirmation_path(conn, :edit, "some-token"))
      response = html_response(conn, 200)
      assert response =~ "<h1>Confirm account</h1>"

      form_action = Routes.courier_confirmation_path(conn, :update, "some-token")
      assert response =~ "action=\"#{form_action}\""
    end
  end

  describe "POST /couriers/confirm/:token" do
    test "confirms the given token once", %{conn: conn, courier: courier} do
      token =
        extract_courier_token(fn url ->
          Accounts.deliver_courier_confirmation_instructions(courier, url)
        end)

      conn = post(conn, Routes.courier_confirmation_path(conn, :update, token))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "Courier confirmed successfully"
      assert Accounts.get_courier!(courier.id).confirmed_at
      refute get_session(conn, :courier_token)
      assert Repo.all(Accounts.CourierToken) == []

      # When not logged in
      conn = post(conn, Routes.courier_confirmation_path(conn, :update, token))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Courier confirmation link is invalid or it has expired"

      # When logged in
      conn =
        build_conn()
        |> log_in_courier(courier)
        |> post(Routes.courier_confirmation_path(conn, :update, token))

      assert redirected_to(conn) == "/"
      refute get_flash(conn, :error)
    end

    test "does not confirm email with invalid token", %{conn: conn, courier: courier} do
      conn = post(conn, Routes.courier_confirmation_path(conn, :update, "oops"))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Courier confirmation link is invalid or it has expired"
      refute Accounts.get_courier!(courier.id).confirmed_at
    end
  end
end
