defmodule VoltWeb.CourierSessionControllerTest do
  use VoltWeb.ConnCase, async: true

  import Volt.AccountsFixtures

  setup do
    %{courier: courier_fixture()}
  end

  describe "GET /couriers/log_in" do
    test "renders log in page", %{conn: conn} do
      conn = get(conn, Routes.courier_session_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Log in</h1>"
      assert response =~ "Register</a>"
      assert response =~ "Forgot your password?</a>"
    end

    test "redirects if already logged in", %{conn: conn, courier: courier} do
      conn = conn |> log_in_courier(courier) |> get(Routes.courier_session_path(conn, :new))
      assert redirected_to(conn) == "/couriers/dashboard"
    end
  end

  describe "POST /couriers/log_in" do
    test "logs the courier in", %{conn: conn, courier: courier} do
      conn =
        post(conn, Routes.courier_session_path(conn, :create), %{
          "courier" => %{"email" => courier.email, "password" => valid_courier_password()}
        })

      assert get_session(conn, :courier_token)
      assert redirected_to(conn) == "/couriers/dashboard"

      # Now do a logged in request and assert on the menu
      conn = get(conn, "/")
      response = html_response(conn, 200)
      assert response =~ courier.email
      assert response =~ "Settings</a>"
      assert response =~ "Log out</a>"
    end

    test "logs the courier in with remember me", %{conn: conn, courier: courier} do
      conn =
        post(conn, Routes.courier_session_path(conn, :create), %{
          "courier" => %{
            "email" => courier.email,
            "password" => valid_courier_password(),
            "remember_me" => "true"
          }
        })

      assert conn.resp_cookies["_volt_web_courier_remember_me"]
      assert redirected_to(conn) == "/couriers/dashboard"
    end

    test "logs the courier in with return to", %{conn: conn, courier: courier} do
      conn =
        conn
        |> init_test_session(courier_return_to: "/foo/bar")
        |> post(Routes.courier_session_path(conn, :create), %{
          "courier" => %{
            "email" => courier.email,
            "password" => valid_courier_password()
          }
        })

      assert redirected_to(conn) == "/foo/bar"
    end

    test "emits error message with invalid credentials", %{conn: conn, courier: courier} do
      conn =
        post(conn, Routes.courier_session_path(conn, :create), %{
          "courier" => %{"email" => courier.email, "password" => "invalid_password"}
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Log in</h1>"
      assert response =~ "Invalid email or password"
    end
  end

  describe "DELETE /couriers/log_out" do
    test "logs the courier out", %{conn: conn, courier: courier} do
      conn = conn |> log_in_courier(courier) |> delete(Routes.courier_session_path(conn, :delete))
      assert redirected_to(conn) == "/"
      refute get_session(conn, :courier_token)
      assert get_flash(conn, :info) =~ "Logged out successfully"
    end

    test "succeeds even if the courier is not logged in", %{conn: conn} do
      conn = delete(conn, Routes.courier_session_path(conn, :delete))
      assert redirected_to(conn) == "/"
      refute get_session(conn, :courier_token)
      assert get_flash(conn, :info) =~ "Logged out successfully"
    end
  end
end
