defmodule VoltWeb.RestaurantAuthTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  alias VoltWeb.RestaurantAuth
  import Volt.AccountsFixtures

  @remember_me_cookie "_volt_web_restaurant_remember_me"

  setup %{conn: conn} do
    conn =
      conn
      |> Map.replace!(:secret_key_base, VoltWeb.Endpoint.config(:secret_key_base))
      |> init_test_session(%{})

    %{restaurant: restaurant_fixture(), conn: conn}
  end

  describe "log_in_restaurant/3" do
    test "stores the restaurant token in the session", %{conn: conn, restaurant: restaurant} do
      conn = RestaurantAuth.log_in_restaurant(conn, restaurant)
      assert token = get_session(conn, :restaurant_token)
      assert get_session(conn, :live_socket_id) == "restaurants_sessions:#{Base.url_encode64(token)}"
      assert redirected_to(conn) == "/restaurants/dashboard"
      assert Accounts.get_restaurant_by_session_token(token)
    end

    test "clears everything previously stored in the session", %{conn: conn, restaurant: restaurant} do
      conn = conn |> put_session(:to_be_removed, "value") |> RestaurantAuth.log_in_restaurant(restaurant)
      refute get_session(conn, :to_be_removed)
    end

    test "redirects to the configured path", %{conn: conn, restaurant: restaurant} do
      conn = conn |> put_session(:restaurant_return_to, "/hello") |> RestaurantAuth.log_in_restaurant(restaurant)
      assert redirected_to(conn) == "/hello"
    end

    test "writes a cookie if remember_me is configured", %{conn: conn, restaurant: restaurant} do
      conn = conn |> fetch_cookies() |> RestaurantAuth.log_in_restaurant(restaurant, %{"remember_me" => "true"})
      assert get_session(conn, :restaurant_token) == conn.cookies[@remember_me_cookie]

      assert %{value: signed_token, max_age: max_age} = conn.resp_cookies[@remember_me_cookie]
      assert signed_token != get_session(conn, :restaurant_token)
      assert max_age == 5_184_000
    end
  end

  describe "logout_restaurant/1" do
    test "erases session and cookies", %{conn: conn, restaurant: restaurant} do
      restaurant_token = Accounts.generate_restaurant_session_token(restaurant)

      conn =
        conn
        |> put_session(:restaurant_token, restaurant_token)
        |> put_req_cookie(@remember_me_cookie, restaurant_token)
        |> fetch_cookies()
        |> RestaurantAuth.log_out_restaurant()

      refute get_session(conn, :restaurant_token)
      refute conn.cookies[@remember_me_cookie]
      assert %{max_age: 0} = conn.resp_cookies[@remember_me_cookie]
      assert redirected_to(conn) == "/"
      refute Accounts.get_restaurant_by_session_token(restaurant_token)
    end

    test "broadcasts to the given live_socket_id", %{conn: conn} do
      live_socket_id = "restaurants_sessions:abcdef-token"
      VoltWeb.Endpoint.subscribe(live_socket_id)

      conn
      |> put_session(:live_socket_id, live_socket_id)
      |> RestaurantAuth.log_out_restaurant()

      assert_receive %Phoenix.Socket.Broadcast{event: "disconnect", topic: ^live_socket_id}
    end

    test "works even if restaurant is already logged out", %{conn: conn} do
      conn = conn |> fetch_cookies() |> RestaurantAuth.log_out_restaurant()
      refute get_session(conn, :restaurant_token)
      assert %{max_age: 0} = conn.resp_cookies[@remember_me_cookie]
      assert redirected_to(conn) == "/"
    end
  end

  describe "fetch_current_restaurant/2" do
    test "authenticates restaurant from session", %{conn: conn, restaurant: restaurant} do
      restaurant_token = Accounts.generate_restaurant_session_token(restaurant)
      conn = conn |> put_session(:restaurant_token, restaurant_token) |> RestaurantAuth.fetch_current_restaurant([])
      assert conn.assigns.current_restaurant.id == restaurant.id
    end

    test "authenticates restaurant from cookies", %{conn: conn, restaurant: restaurant} do
      logged_in_conn =
        conn |> fetch_cookies() |> RestaurantAuth.log_in_restaurant(restaurant, %{"remember_me" => "true"})

      restaurant_token = logged_in_conn.cookies[@remember_me_cookie]
      %{value: signed_token} = logged_in_conn.resp_cookies[@remember_me_cookie]

      conn =
        conn
        |> put_req_cookie(@remember_me_cookie, signed_token)
        |> RestaurantAuth.fetch_current_restaurant([])

      assert get_session(conn, :restaurant_token) == restaurant_token
      assert conn.assigns.current_restaurant.id == restaurant.id
    end

    test "does not authenticate if data is missing", %{conn: conn, restaurant: restaurant} do
      _ = Accounts.generate_restaurant_session_token(restaurant)
      conn = RestaurantAuth.fetch_current_restaurant(conn, [])
      refute get_session(conn, :restaurant_token)
      refute conn.assigns.current_restaurant
    end
  end

  describe "redirect_if_restaurant_is_authenticated/2" do
    test "redirects if restaurant is authenticated", %{conn: conn, restaurant: restaurant} do
      conn = conn |> assign(:current_restaurant, restaurant) |> RestaurantAuth.redirect_if_restaurant_is_authenticated([])
      assert conn.halted
      assert redirected_to(conn) == "/restaurants/dashboard"
    end

    test "does not redirect if restaurant is not authenticated", %{conn: conn} do
      conn = RestaurantAuth.redirect_if_restaurant_is_authenticated(conn, [])
      refute conn.halted
      refute conn.status
    end
  end

  describe "require_authenticated_restaurant/2" do
    test "redirects if restaurant is not authenticated", %{conn: conn} do
      conn = conn |> fetch_flash() |> RestaurantAuth.require_authenticated_restaurant([])
      assert conn.halted
      assert redirected_to(conn) == Routes.restaurant_session_path(conn, :new)
      assert get_flash(conn, :error) == "You must log in to access this page."
    end

    test "stores the path to redirect to on GET", %{conn: conn} do
      halted_conn =
        %{conn | path_info: ["foo"], query_string: ""}
        |> fetch_flash()
        |> RestaurantAuth.require_authenticated_restaurant([])

      assert halted_conn.halted
      assert get_session(halted_conn, :restaurant_return_to) == "/foo"

      halted_conn =
        %{conn | path_info: ["foo"], query_string: "bar=baz"}
        |> fetch_flash()
        |> RestaurantAuth.require_authenticated_restaurant([])

      assert halted_conn.halted
      assert get_session(halted_conn, :restaurant_return_to) == "/foo?bar=baz"

      halted_conn =
        %{conn | path_info: ["foo"], query_string: "bar", method: "POST"}
        |> fetch_flash()
        |> RestaurantAuth.require_authenticated_restaurant([])

      assert halted_conn.halted
      refute get_session(halted_conn, :restaurant_return_to)
    end

    test "does not redirect if restaurant is authenticated", %{conn: conn, restaurant: restaurant} do
      conn = conn |> assign(:current_restaurant, restaurant) |> RestaurantAuth.require_authenticated_restaurant([])
      refute conn.halted
      refute conn.status
    end
  end
end
