defmodule VoltWeb.RestaurantConfirmationControllerTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  alias Volt.Repo
  import Volt.AccountsFixtures

  setup do
    %{restaurant: restaurant_fixture()}
  end

  describe "GET /restaurants/confirm" do
    test "renders the resend confirmation page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_confirmation_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Resend confirmation instructions</h1>"
    end
  end

  describe "POST /restaurants/confirm" do
    @tag :capture_log
    test "sends a new confirmation token", %{conn: conn, restaurant: restaurant} do
      conn =
        post(conn, Routes.restaurant_confirmation_path(conn, :create), %{
          "restaurant" => %{"email" => restaurant.email}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.get_by!(Accounts.RestaurantToken, restaurant_id: restaurant.id).context == "confirm"
    end

    test "does not send confirmation token if Restaurant is confirmed", %{conn: conn, restaurant: restaurant} do
      Repo.update!(Accounts.Restaurant.confirm_changeset(restaurant))

      conn =
        post(conn, Routes.restaurant_confirmation_path(conn, :create), %{
          "restaurant" => %{"email" => restaurant.email}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      refute Repo.get_by(Accounts.RestaurantToken, restaurant_id: restaurant.id)
    end

    test "does not send confirmation token if email is invalid", %{conn: conn} do
      conn =
        post(conn, Routes.restaurant_confirmation_path(conn, :create), %{
          "restaurant" => %{"email" => "unknown@example.com"}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.all(Accounts.RestaurantToken) == []
    end
  end

  describe "GET /restaurants/confirm/:token" do
    test "renders the confirmation page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_confirmation_path(conn, :edit, "some-token"))
      response = html_response(conn, 200)
      assert response =~ "<h1>Confirm account</h1>"

      form_action = Routes.restaurant_confirmation_path(conn, :update, "some-token")
      assert response =~ "action=\"#{form_action}\""
    end
  end

  describe "POST /restaurants/confirm/:token" do
    test "confirms the given token once", %{conn: conn, restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_confirmation_instructions(restaurant, url)
        end)

      conn = post(conn, Routes.restaurant_confirmation_path(conn, :update, token))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "Restaurant confirmed successfully"
      assert Accounts.get_restaurant!(restaurant.id).confirmed_at
      refute get_session(conn, :restaurant_token)
      assert Repo.all(Accounts.RestaurantToken) == []

      # When not logged in
      conn = post(conn, Routes.restaurant_confirmation_path(conn, :update, token))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Restaurant confirmation link is invalid or it has expired"

      # When logged in
      conn =
        build_conn()
        |> log_in_restaurant(restaurant)
        |> post(Routes.restaurant_confirmation_path(conn, :update, token))

      assert redirected_to(conn) == "/"
      refute get_flash(conn, :error)
    end

    test "does not confirm email with invalid token", %{conn: conn, restaurant: restaurant} do
      conn = post(conn, Routes.restaurant_confirmation_path(conn, :update, "oops"))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Restaurant confirmation link is invalid or it has expired"
      refute Accounts.get_restaurant!(restaurant.id).confirmed_at
    end
  end
end
