defmodule VoltWeb.RestaurantDashboardControllerTest do
  use VoltWeb.ConnCase
  import Volt.AccountsFixtures
  require Logger
  alias Volt.Sales.Order
  alias Volt.Repo

  defp init_order(status) do
    customer = customer_fixture(%{
      address: "Raatuse 22",
      balance: 0.00,
      birth_date: ~D[2022-11-04],
      card_number: "123123123123123",
      email: "roro@gmail.com",
      password: "roromain12345*",
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: "+33641941064",
      city: "Tartu",
      zip_code: "51009"
    })
    restaurant = restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "kolmtilli@gmail.com",
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: "kolmtilli12345*",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 1
    })
    order = %Order{order_status: status, order_price: 10.00, total_price: 12.50}
    order = Ecto.build_assoc(restaurant, :orders, order)
    order = Ecto.build_assoc(customer, :orders, order)
    {_, order} = Repo.insert(order)
    {order, restaurant, customer}
  end

  defp init_order_courier() do
    customer = customer_fixture(%{
      address: "Raatuse 22",
      balance: 0.00,
      birth_date: ~D[2022-11-04],
      card_number: "123123123123123",
      email: "roro@gmail.com",
      password: "roromain12345*",
      first_name: "some first_name",
      last_name: "some last_name",
      phone_number: "+33641941064",
      city: "Tartu",
      zip_code: "51009"
    })
    restaurant = restaurant_fixture(%{
      address: "Kastani 42",
      closing_time: ~T[14:00:00],
      email: "kolmtilli@gmail.com",
      first_name: "some first_name",
      last_name: "some last_name",
      name: "some name",
      opening_time: ~T[14:00:00],
      password: "kolmtilli12345*",
      phone_number: "+33641941068",
      city: "Tartu",
      zip_code: "50410",
      price_level: 1
    })
    courier = courier_fixture()

    prep_duration = 10
    prep_time = NaiveDateTime.add(NaiveDateTime.local_now(), prep_duration*60, :second)
    del_duration = 20
    del_time = NaiveDateTime.add(prep_time, del_duration*60, :second)

    order = %Order{order_status: "IN_PROCESS", order_price: 10.00, total_price: 12.50, preparation_duration: prep_duration, preparation_time: prep_time, delivery_duration: del_duration, delivery_time: del_time}
    order = Ecto.build_assoc(restaurant, :orders, order)
    order = Ecto.build_assoc(customer, :orders, order)
    order = Ecto.build_assoc(courier, :orders, order)
    {_, order} = Repo.insert(order)
    {order, restaurant, customer, courier}
  end

  describe "index" do
    test "list pending orders", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_dashboard_path(conn, :index))
      assert html_response(conn, 200) =~ "Pending Orders"
    end

    test "list in process orders", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_dashboard_path(conn, :index))
      assert html_response(conn, 200) =~ "In Process Orders"
    end
  end

  describe "accept" do
    test "render acceptance page", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> get(Routes.restaurant_dashboard_path(conn, :show_accept, order.id), res)
      assert html_response(conn, 200) =~ "Accept order"
    end

    test "accept pending order (confirmation)", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id, "order" => %{"preparation_duration" => "20"}}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :accept, order.id), res)
      assert redirected_to(conn) == "/restaurants/dashboard"
      assert get_flash(conn, :info) =~ "Order accepted successfully."
    end

    test "accept pending order (rejection with no preparation duration or null)", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id, "order" => %{"preparation_duration" => ""}}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :accept, order.id), res)
      response = html_response(conn, 200)
      assert response =~ "Accept order"
      assert response =~ "You must provide a preparation duration estimation (minute)!"
    end
  end

  describe "cancel" do
    test "render cancel page", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> get(Routes.restaurant_dashboard_path(conn, :show_cancel, order.id), res)
      assert html_response(conn, 200) =~ "Cancel order"
    end

    test "cancel order with error", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id, "order" => %{"restaurant_message" => ""}}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :cancel, order.id), res)
      response = html_response(conn, 200)
      assert response =~ "Cancel order"
      assert response =~ "You must provide a message"
    end

    test "cancel order with confirmation", %{conn: conn} do
      {order, restaurant, _customer} = init_order("PENDING")
      res = %{"id" => order.id, "order" => %{"restaurant_message" => "My random message"}}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :cancel, order.id), res)
      assert redirected_to(conn) == "/restaurants/dashboard"
      assert get_flash(conn, :info) =~ "Order canceled successfully."
    end
  end

  describe "prepare order" do
    test "mark in process order as prepared (success with courier)", %{conn: conn} do
      {order, restaurant, _customer, _courier} = init_order_courier()
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :prepared, order.id), res)
      assert redirected_to(conn) == "/restaurants/dashboard"
      assert get_flash(conn, :info) =~ "Order marked as prepared successfully."
    end

    test "mark in process order as prepared (error without courier)", %{conn: conn} do
      {order, restaurant, _customer} = init_order("IN_PROCESS")
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> put(Routes.restaurant_dashboard_path(conn, :prepared, order.id), res)
      assert redirected_to(conn) == "/restaurants/dashboard"
      assert get_flash(conn, :error) =~ "Order has to have a courier first!"
    end
  end

  describe "Prepared Orders history" do
    test "Get history of prepared orders", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_dashboard_path(conn, :history))
      assert html_response(conn, 200) =~ "Prepared Orders"
    end

    test "Get prepared order information", %{conn: conn} do
      {order, restaurant, _customer, _courier} = init_order_courier()
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> get(Routes.restaurant_dashboard_path(conn, :prepared_info, order.id), res)
      assert html_response(conn, 200) =~ "Info"
    end
  end

  describe "Manage Order" do
    test "get order management page", %{conn: conn} do
      {order, restaurant, _customer, _courier} = init_order_courier()
      res = %{"id" => order.id}
      conn = conn |> log_in_restaurant(restaurant) |> get(Routes.restaurant_dashboard_path(conn, :show_manage, order.id), res)
      assert html_response(conn, 200) =~ "<h1>Manage order</h1>"
    end
  end
end
