defmodule VoltWeb.ItemControllerTest do
  use VoltWeb.ConnCase

  import Volt.SalesFixtures
  import Volt.AccountsFixtures

  @create_attrs %{description: "some description", name: "some name", unit_price: "12.99", discount: "0"}
  @update_attrs %{description: "some updated description", name: "some updated name", unit_price: "13", discount: "20"}
  @invalid_attrs %{description: nil, name: nil, unit_price: nil, discount: "101"}

  describe "index" do
    test "lists all items", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_item_path(conn, :index))
      assert html_response(conn, 200) =~ "Menu"
    end
  end

  describe "new item" do
    test "renders form", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_item_path(conn, :new))
      assert html_response(conn, 200) =~ "New Item"
    end
  end

  describe "create item" do
    test "redirects to show when data is valid", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = post(conn, Routes.restaurant_item_path(conn, :create), item: @create_attrs)

      assert %{id: id} = redirected_params(conn)
      assert redirected_to(conn) == Routes.restaurant_item_path(conn, :show, id)

      conn = get(conn, Routes.restaurant_item_path(conn, :show, id))
      assert html_response(conn, 200) =~ "Show Item"
    end

    test "renders errors when data is invalid", %{conn: conn} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = post(conn, Routes.restaurant_item_path(conn, :create), item: @invalid_attrs)
      assert html_response(conn, 200) =~ "New Item"
    end
  end

  describe "edit item" do
    setup [:create_item]

    test "renders form for editing chosen item", %{conn: conn, item: item} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = get(conn, Routes.restaurant_item_path(conn, :edit, item))
      assert html_response(conn, 200) =~ "Edit Item"
    end
  end

  describe "update item" do
    setup [:create_item]

    test "redirects when data is valid", %{conn: conn, item: item} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = put(conn, Routes.restaurant_item_path(conn, :update, item), item: @update_attrs)
      assert redirected_to(conn) == Routes.restaurant_item_path(conn, :show, item)

      conn = get(conn, Routes.restaurant_item_path(conn, :show, item))
      assert html_response(conn, 200) =~ "some updated description"
    end

    test "renders errors when data is invalid", %{conn: conn, item: item} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = put(conn, Routes.restaurant_item_path(conn, :update, item), item: @invalid_attrs)
      assert html_response(conn, 200) =~ "Edit Item"
    end
  end

  describe "delete item" do
    setup [:create_item]

    test "deletes chosen item", %{conn: conn, item: item} do
      conn = conn |> log_in_restaurant(restaurant_fixture()) |> get(Routes.restaurant_registration_path(conn, :new))
      conn = delete(conn, Routes.restaurant_item_path(conn, :delete, item))
      assert redirected_to(conn) == Routes.restaurant_item_path(conn, :index)

      assert_error_sent 500, fn ->
        get(conn, Routes.restaurant_item_path(conn, :show, item))
      end
    end
  end

  defp create_item(_) do
    item = item_fixture()
    %{item: item}
  end
end
