defmodule VoltWeb.RestaurantResetPasswordControllerTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  alias Volt.Repo
  import Volt.AccountsFixtures

  setup do
    %{restaurant: restaurant_fixture()}
  end

  describe "GET /restaurants/reset_password" do
    test "renders the reset password page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_reset_password_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Forgot your password?</h1>"
    end
  end

  describe "POST /restaurants/reset_password" do
    @tag :capture_log
    test "sends a new reset password token", %{conn: conn, restaurant: restaurant} do
      conn =
        post(conn, Routes.restaurant_reset_password_path(conn, :create), %{
          "restaurant" => %{"email" => restaurant.email}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.get_by!(Accounts.RestaurantToken, restaurant_id: restaurant.id).context == "reset_password"
    end

    test "does not send reset password token if email is invalid", %{conn: conn} do
      conn =
        post(conn, Routes.restaurant_reset_password_path(conn, :create), %{
          "restaurant" => %{"email" => "unknown@example.com"}
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :info) =~ "If your email is in our system"
      assert Repo.all(Accounts.RestaurantToken) == []
    end
  end

  describe "GET /restaurants/reset_password/:token" do
    setup %{restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_reset_password_instructions(restaurant, url)
        end)

      %{token: token}
    end

    test "renders reset password", %{conn: conn, token: token} do
      conn = get(conn, Routes.restaurant_reset_password_path(conn, :edit, token))
      assert html_response(conn, 200) =~ "<h1>Reset password</h1>"
    end

    test "does not render reset password with invalid token", %{conn: conn} do
      conn = get(conn, Routes.restaurant_reset_password_path(conn, :edit, "oops"))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Reset password link is invalid or it has expired"
    end
  end

  describe "PUT /restaurants/reset_password/:token" do
    setup %{restaurant: restaurant} do
      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_reset_password_instructions(restaurant, url)
        end)

      %{token: token}
    end

    test "resets password once", %{conn: conn, restaurant: restaurant, token: token} do
      conn =
        put(conn, Routes.restaurant_reset_password_path(conn, :update, token), %{
          "restaurant" => %{
            "password" => "new valid password",
            "password_confirmation" => "new valid password"
          }
        })

      assert redirected_to(conn) == Routes.restaurant_session_path(conn, :new)
      refute get_session(conn, :restaurant_token)
      assert get_flash(conn, :info) =~ "Password reset successfully"
      assert Accounts.get_restaurant_by_email_and_password(restaurant.email, "new valid password")
    end

    test "does not reset password on invalid data", %{conn: conn, token: token} do
      conn =
        put(conn, Routes.restaurant_reset_password_path(conn, :update, token), %{
          "restaurant" => %{
            "password" => "too short",
            "password_confirmation" => "does not match"
          }
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Reset password</h1>"
      assert response =~ "should be at least 12 character(s)"
      assert response =~ "does not match password"
    end

    test "does not reset password with invalid token", %{conn: conn} do
      conn = put(conn, Routes.restaurant_reset_password_path(conn, :update, "oops"))
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) =~ "Reset password link is invalid or it has expired"
    end
  end
end
