defmodule VoltWeb.RestaurantSessionControllerTest do
  use VoltWeb.ConnCase, async: true

  import Volt.AccountsFixtures

  setup do
    %{restaurant: restaurant_fixture()}
  end

  describe "GET /restaurants/log_in" do
    test "renders log in page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_session_path(conn, :new))
      response = html_response(conn, 200)
      assert response =~ "<h1>Log in</h1>"
      assert response =~ "Register</a>"
      assert response =~ "Forgot your password?</a>"
    end

    test "redirects if already logged in", %{conn: conn, restaurant: restaurant} do
      conn = conn |> log_in_restaurant(restaurant) |> get(Routes.restaurant_session_path(conn, :new))
      assert redirected_to(conn) == "/restaurants/dashboard"
    end
  end

  describe "POST /restaurants/log_in" do
    test "logs the restaurant in", %{conn: conn, restaurant: restaurant} do
      conn =
        post(conn, Routes.restaurant_session_path(conn, :create), %{
          "restaurant" => %{"email" => restaurant.email, "password" => valid_restaurant_password()}
        })

      assert get_session(conn, :restaurant_token)
      assert redirected_to(conn) == "/restaurants/dashboard"

      # Now do a logged in request and assert on the menu
      conn = get(conn, "/")
      response = html_response(conn, 200)
      assert response =~ restaurant.email
      assert response =~ "Settings</a>"
      assert response =~ "Log out</a>"
    end

    test "logs the restaurant in with remember me", %{conn: conn, restaurant: restaurant} do
      conn =
        post(conn, Routes.restaurant_session_path(conn, :create), %{
          "restaurant" => %{
            "email" => restaurant.email,
            "password" => valid_restaurant_password(),
            "remember_me" => "true"
          }
        })

      assert conn.resp_cookies["_volt_web_restaurant_remember_me"]
      assert redirected_to(conn) == "/restaurants/dashboard"
    end

    test "logs the restaurant in with return to", %{conn: conn, restaurant: restaurant} do
      conn =
        conn
        |> init_test_session(restaurant_return_to: "/foo/bar")
        |> post(Routes.restaurant_session_path(conn, :create), %{
          "restaurant" => %{
            "email" => restaurant.email,
            "password" => valid_restaurant_password()
          }
        })

      assert redirected_to(conn) == "/foo/bar"
    end

    test "emits error message with invalid credentials", %{conn: conn, restaurant: restaurant} do
      conn =
        post(conn, Routes.restaurant_session_path(conn, :create), %{
          "restaurant" => %{"email" => restaurant.email, "password" => "invalid_password"}
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Log in</h1>"
      assert response =~ "Invalid email or password"
    end
  end

  describe "DELETE /restaurants/log_out" do
    test "logs the restaurant out", %{conn: conn, restaurant: restaurant} do
      conn = conn |> log_in_restaurant(restaurant) |> delete(Routes.restaurant_session_path(conn, :delete))
      assert redirected_to(conn) == "/"
      refute get_session(conn, :restaurant_token)
      assert get_flash(conn, :info) =~ "Logged out successfully"
    end

    test "succeeds even if the restaurant is not logged in", %{conn: conn} do
      conn = delete(conn, Routes.restaurant_session_path(conn, :delete))
      assert redirected_to(conn) == "/"
      refute get_session(conn, :restaurant_token)
      assert get_flash(conn, :info) =~ "Logged out successfully"
    end
  end
end
