defmodule VoltWeb.RestaurantSettingsControllerTest do
  use VoltWeb.ConnCase, async: true

  alias Volt.Accounts
  import Volt.AccountsFixtures

  setup :register_and_log_in_restaurant

  describe "GET /restaurants/settings" do
    test "renders settings page", %{conn: conn} do
      conn = get(conn, Routes.restaurant_settings_path(conn, :edit))
      response = html_response(conn, 200)
      assert response =~ "<h1>Settings</h1>"
    end

    test "redirects if restaurant is not logged in" do
      conn = build_conn()
      conn = get(conn, Routes.restaurant_settings_path(conn, :edit))
      assert redirected_to(conn) == Routes.restaurant_session_path(conn, :new)
    end
  end

  describe "PUT /restaurants/settings (change password form)" do
    test "updates the restaurant password and resets tokens", %{conn: conn, restaurant: restaurant} do
      new_password_conn =
        put(conn, Routes.restaurant_settings_path(conn, :update), %{
          "action" => "update_password",
          "current_password" => valid_restaurant_password(),
          "restaurant" => %{
            "password" => "new valid password",
            "password_confirmation" => "new valid password"
          }
        })

      assert redirected_to(new_password_conn) == Routes.restaurant_settings_path(conn, :edit)
      assert get_session(new_password_conn, :restaurant_token) != get_session(conn, :restaurant_token)
      assert get_flash(new_password_conn, :info) =~ "Password updated successfully"
      assert Accounts.get_restaurant_by_email_and_password(restaurant.email, "new valid password")
    end

    test "does not update password on invalid data", %{conn: conn} do
      old_password_conn =
        put(conn, Routes.restaurant_settings_path(conn, :update), %{
          "action" => "update_password",
          "current_password" => "invalid",
          "restaurant" => %{
            "password" => "too short",
            "password_confirmation" => "does not match"
          }
        })

      response = html_response(old_password_conn, 200)
      assert response =~ "<h1>Settings</h1>"
      assert response =~ "should be at least 12 character(s)"
      assert response =~ "does not match password"
      assert response =~ "is not valid"

      assert get_session(old_password_conn, :restaurant_token) == get_session(conn, :restaurant_token)
    end
  end

  describe "PUT /restaurants/settings (change email form)" do
    @tag :capture_log
    test "updates the restaurant email", %{conn: conn, restaurant: restaurant} do
      conn =
        put(conn, Routes.restaurant_settings_path(conn, :update), %{
          "action" => "update_email",
          "current_password" => valid_restaurant_password(),
          "restaurant" => %{"email" => unique_restaurant_email()}
        })

      assert redirected_to(conn) == Routes.restaurant_settings_path(conn, :edit)
      assert get_flash(conn, :info) =~ "A link to confirm your email"
      assert Accounts.get_restaurant_by_email(restaurant.email)
    end

    test "does not update email on invalid data", %{conn: conn} do
      conn =
        put(conn, Routes.restaurant_settings_path(conn, :update), %{
          "action" => "update_email",
          "current_password" => "invalid",
          "restaurant" => %{"email" => "with spaces"}
        })

      response = html_response(conn, 200)
      assert response =~ "<h1>Settings</h1>"
      assert response =~ "must have the @ sign and no spaces"
      assert response =~ "is not valid"
    end
  end

  describe "GET /restaurants/settings/confirm_email/:token" do
    setup %{restaurant: restaurant} do
      email = unique_restaurant_email()

      token =
        extract_restaurant_token(fn url ->
          Accounts.deliver_restaurant_update_email_instructions(%{restaurant | email: email}, restaurant.email, url)
        end)

      %{token: token, email: email}
    end

    test "updates the restaurant email once", %{conn: conn, restaurant: restaurant, token: token, email: email} do
      conn = get(conn, Routes.restaurant_settings_path(conn, :confirm_email, token))
      assert redirected_to(conn) == Routes.restaurant_settings_path(conn, :edit)
      assert get_flash(conn, :info) =~ "Email changed successfully"
      refute Accounts.get_restaurant_by_email(restaurant.email)
      assert Accounts.get_restaurant_by_email(email)

      conn = get(conn, Routes.restaurant_settings_path(conn, :confirm_email, token))
      assert redirected_to(conn) == Routes.restaurant_settings_path(conn, :edit)
      assert get_flash(conn, :error) =~ "Email change link is invalid or it has expired"
    end

    test "does not update email with invalid token", %{conn: conn, restaurant: restaurant} do
      conn = get(conn, Routes.restaurant_settings_path(conn, :confirm_email, "oops"))
      assert redirected_to(conn) == Routes.restaurant_settings_path(conn, :edit)
      assert get_flash(conn, :error) =~ "Email change link is invalid or it has expired"
      assert Accounts.get_restaurant_by_email(restaurant.email)
    end

    test "redirects if restaurant is not logged in", %{token: token} do
      conn = build_conn()
      conn = get(conn, Routes.restaurant_settings_path(conn, :confirm_email, token))
      assert redirected_to(conn) == Routes.restaurant_session_path(conn, :new)
    end
  end
end
